Package: FAwR
Type: Package
Title: Functions and Datasets for "Forest Analytics with R"
Version: 1.2.0
Date: 2025-06-22
Maintainer: Andrew Robinson <apro@unimelb.edu.au>
Depends: R (>= 2.10), MASS, lattice
Description: Provides functions and datasets from the book "Forest Analytics with R".
License: GPL-3
LazyLoad: yes
Authors@R: c(person(given = "Andrew",
                    family = "Robinson",
                    role = c("aut", "cre"),
                    email = "apro@unimelb.edu.au"),
             person(given = "Jeff",
                    family = "Hamann",
                    role = "aut"))
NeedsCompilation: no
Packaged: 2025-06-22 07:21:20 UTC; root
Author: Andrew Robinson [aut, cre],
  Jeff Hamann [aut]
Repository: CRAN
Date/Publication: 2025-06-22 09:40:05 UTC
Built: R 4.6.0; ; 2026-04-26 17:56:58 UTC; unix

importFrom("graphics", "abline", "plot", "text")
importFrom("stats", "uniroot")
importFrom("utils", "tail")

import("MASS")
import("lattice")

export("SSallometric","show.cols.with.na",
       "generate.log.vols","ht.fvs.ni.m","sp.sums.2","vol.fvs.ni.m3")

# $Id: ratio_summary.R 169 2006-01-05 05:02:44Z andrewr $

rm(list=ls())

load("../scripts/booted_ratio.RData")
require(lattice)

opar <- par(mfrow=c(3,3), mar=c(5,4,0,0))
for (i in 1:length(levels(plots$forest))) {
  plot(density(plots$ba[plots$forest==unique(plots$forest)[i]]),
       main="", ylab="",
       xlab=paste("National forest ID:", unique(plots$forest)[i]))
}
par(opar)

opar <- par(mfrow=c(3,3), mar=c(5,4,0,0))
for (i in 1:length(levels(plots$forest))) {
  qqnorm(plots$ba[plots$forest==unique(plots$forest)[i]],
         main="", ylab="",
         xlab=paste("National forest ID:", unique(plots$forest)[i]))
  qqline(plots$ba[plots$forest==unique(plots$forest)[i]])
}
par(opar)

opar <- par(mfrow=c(3,3), mar=c(5,4,0,0))
for (i in 1:length(levels(plots$forest))) {
  ratio.res <- 
  qqnorm(plots$ba[plots$forest==unique(plots$forest)[i]],
         main="", ylab="",
         xlab=paste("National forest ID:", unique(plots$forest)[i]))
  qqline(plots$ba[plots$forest==unique(plots$forest)[i]])
}
par(opar)

xyplot(ba ~ ht | forest,
       panel = function(x, y) {
         panel.xyplot(x, y)
         panel.loess(x,y, span=1)
       },
       data=plots)

sapply(overall.results, class)
sapply(results, class)

overall.results$interval <-
  factor(overall.results$interval, levels=unique(overall.results$interval))
results$interval <-
  factor(results$interval, levels=unique(results$interval))



means <- tapply(plots$ba, plots$forest, mean)

xyplot(contained ~ n | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       data=overall.results)

xyplot(length ~ n | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
       },
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       data=overall.results)

xyplot(contained ~ length | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       data=overall.results)

#########################################  BY FOREST


xyplot(length ~ n | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=2, title="Nearest Forest",
         corner=c(1,1), x=0.96, y=0.71),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
       },
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       data=results)


xyplot(contained ~ n | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=2, title="Nearest Forest",
         corner=c(1,1), x=0.96, y=0.71),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
         panel.abline(h=0.95, col="darkgrey")
       },
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       data=results)



xyplot(contained ~ length | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=2, title="Nearest Forest",
         corner=c(1,1), x=0.96, y=0.71),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
         panel.abline(h=0.95, col="darkgrey")
       },
       skip=c(rep(FALSE, 11), TRUE, rep(FALSE, 3)),
       index.cond=list(c(1:3,7,4:6,8,9:14)),
       data=results)


require(e1071)

ks.statistic <- function(x, y, ...,
                 alternative = c("two.sided", "less", "greater"),
                 exact = NULL) 
  ks.test(x, y, ...,
          alternative = c("two.sided", "less", "greater"),
          exact = NULL)$statistic
  
by.forest <- aggregate(x = list(ks.ba=plots$ba),
                       by = list(forest=plots$forest),
                       FUN = ks.statistic,
                       y="pnorm")

results.temp <- results[results$interval %in% c("classical","linearized",
                                                "jackknife","studentized"),]
results.temp$interval <- factor(results.temp$interval)

my.trellis.pars <- trellis.par.get()
my.trellis.pars$superpose.line$lty <- 1:8
my.trellis.pars$superpose.line$col <- rep("black", 8)
trellis.par.set(my.trellis.pars)

xyplot(contained ~ n | forest,
       groups=interval,
       auto.key=list(columns=2, lines=TRUE, points=FALSE,
         title="Interval Style",
         text=c("Classical","Linearized","Jackknife","Studentized Bootstrap")),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results.temp$interval))) {
           panel.xyplot(x[groups[subscripts]==
                          levels(results.temp$interval)[i]],
                        y[groups[subscripts]==
                          levels(results.temp$interval)[i]],
                        lty=trellis.par.get()$superpose.line$lty[i],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
         panel.abline(h=0.95, col="darkgrey")
         panel.text(115, 0.962,
                    sprintf("%.3f",
                            with(by.forest,
                                 ks.ba[forest==
                                       results.temp$forest[subscripts][1]])))
       },
       index.cond=list(order(by.forest$ks.ba)),
       data=results.temp)


ks.normal.ratio.residual <- function(x, y) {  
  ks.test(residuals(lm(x ~ y - 1)), y="pnorm")$statistic
}

by.forest <- aggregate(x = list(ks.ba=plots$ba),
                       by = list(forest=plots$forest),
                       FUN = ks.normal.ratio.residual,
                       y = plots$ht)

#	$Id$	

## this is a set of functions to help with the R book
## for harvest scheduling.

##    No.   Row name   St   Activity     Lower bound   Upper bound    Marginal
## ------ ------------ -- ------------- ------------- ------------- -------------
##      1 hvol1        NS             0             0             =       78.3347
##      2 hvol2        NS             0             0             =      -14.3923
##      3 hvol3        NS             0             0             =      -104.408

get.row.report <- function( lp ) {
  
  ## get the number of columns
  nrows <- lpx_get_num_rows( lp )
  
  ## build the table for the rows
  r <- NULL
  for( i in 1:nrows ) {
    r <- rbind( r,
               as.data.frame( cbind( lpx_get_row_name( lp, i ),
                                    lpx_get_row_stat( lp, i ),
                                    ##glpk_strerror(lpx_get_row_stat(lp,i)),
                                    lpx_get_row_prim( lp, i ),
                                    lpx_get_row_lb( lp, i ),
                                    lpx_get_row_ub( lp, i ),
                                    lpx_get_row_dual( lp, i ),
                                    ##lpx_get_row_b_ind( lp, i ),
                                    glpk_strerror(lpx_get_row_type(lp,i))
                                    )
                             )
               )


  }
  
  
  names(r) <- c("name","status","prim","lb","ub","dual","strerr" )
  rownames( r ) <- 1:nrows

  r
}

  

#    No. Column name  St   Activity     Lower bound   Upper bound    Marginal
# ------ ------------ -- ------------- ------------- ------------- -------------
#      1 x(1)         B        109.153             0               
#      2 x(2)         NL             0             0                    -76.3559
#      3 x(3)         B            200             0               


get.col.report <- function( lp ) {

  ncols <- lpx_get_num_cols( lp )

  ## build the table for the columns
  c <- NULL
  for( i in 1:ncols ) {
    c <- rbind( c,
               as.data.frame( cbind( I(lpx_get_col_name( lp, i )),
                                    I(lpx_get_col_stat( lp, i ) ),
                                    I(lpx_get_col_prim( lp, i ) ),
                                    I(lpx_get_col_lb( lp, i ) ),
                                    I(lpx_get_col_ub( lp, i ) ),
                                    I(lpx_get_col_dual( lp, i ) ),
                                    I(lpx_get_col_b_ind( lp, i ) ),
                                    I(lpx_get_col_type( lp, i ) )
                                    )
                             )
               )
  }
  
  
  names(c) <- c("name","status","activity","lb","ub","dual","b_ind","type" )
  rownames( c ) <- 1:ncols

  c
}

#	$Id$	

## this script contains two functions:
## generate.log.vols
## sp.sums.2

## where the function generate.log.vols returns a dataframe object
## that contains the original stem measurements and appends columns
## named log.grades, where log.grades corresponds to the minimum log diameters
## defined by log.breaks.

## this script will generate the log volumes
## and return a data.frame that contains the
## original stem list and the colums of the
## log volumes appended on to the end

## > sample.13$plants
##    plot sp.code         d6          dbh       tht        cr n.stems   expf
## 1     1      PP 10.6968804  8.032913840 30.188408 0.7053927       1 100.00
## 2     1      DF  6.4655181  4.455463746 30.157367 0.9636109       1 100.00
## 3     2      IC  2.6718796  0.653022095  6.602279 0.8462617       1 100.00

## this function is 200 lines long.
## too long for direct insertion in book page. 
## this function will not replace already existing columns
## in a data.frame, but will append them
generate.log.vols <- function( x,
                              log.breaks=c(2,5,12,18,32,999),
                              log.grades=c("pulp","s4","s3","s2","s1","peeler"),
                              display.stems=FALSE )
{
  
  if( !is.data.frame( x ) ) {
    stop( "x needs to be a data.frame" )
  }

  ## allocate the grade volumes data frame that will be
  ## cbind'd to the stems data frame (n1 -- for now)
  grade.vols <- as.data.frame( matrix( rep( 0, nrow(x) ),
                                      nrow(x),
                                      length(log.grades),
                                      byrow=TRUE ) )
  names( grade.vols ) <- log.grades

  ## code up the taper function in here...
  ## function is orginally in inches, but need to work in metric
  ## WAY TOO MUCH BUTT SWELL, MAYBE...
  ## inbound values are in inches
  ## need to convert to centimeters (dbh) and meters (H,hi)
  bcmof.diDBH <- function( dbh, tht, cr, hi ) {

    ##dbh <- dbh / 2.54 ## convert from cm to inches
    ##tht <- tht / 3.2808 ## convert to feet
    ##hi <- hi / 3.2808 ## convert to feet  

    dbh <- dbh * 2.54 ## convert from inches to cm
    tht <- tht * 0.3048 ## convert to meters
    hi <- hi * 0.3048 ## convert to meters 

    ## these numbers should be in what units again?
    ##print( sprintf( "dbh = %f, tht=%f, hi=%f", dbh, tht, hi ) )

    if( hi >= tht ) {
      retval <- 0.0
    } else {
      p <- 0.25    
      dib <- 1.02453 * dbh^0.88809 * 1.00035^dbh
      X <- ( 1.0 - sqrt( hi / tht ) ) / ( 1.0 - sqrt( p ) )
      Z = hi / tht
      a = 0.95086 * Z * Z;
      b = -0.18090 * log( Z + 0.001 );
      c = 0.61407 * sqrt( Z ) +  -0.35105 * exp( Z );
      d = 0.05686 * ( dbh / tht ); 
      retval <- ( dib * X^( a + b + c + d ) )
    }
    retval / 2.54 ## convert from cm to inches
  }
  
  
  ## the md is the merchantable diameter
  merch.height.func <- function( hi, dbh, tht, cr, md ) {
    dib <- bcmof.diDBH( dbh, tht, cr, hi )
    diff <- dib - md
    diff
  }
  
  ## smalian volume
  ## d1, d2, l in meters or feet
  ## return value is in cubic meters or feet
  smal.vol <- function( d1, d2, l ) {
    ##smal.vol <- pi * l * (d1*d1 + d2*d2 ) / 8.0 
    smal.vol <- 0.0054541539 * l * (d1*d1 + d2*d2 ) / 2
    smal.vol
  }
  
  logs <- NULL

  ##print( "starting to buck stem..." )  
  for( s in 1:nrow(x) ) {  
    ##print( x[s,] )    
    ## if dbh == 0, don't attempt to merch
    if( x[s,]$dbh <= 5.0 ) next
    
    ## for each log.brk
    ## determine if the diameter break is smaller
    ## than the diameter inside bark at the stump
    mh.bks <- rep( 0, length( log.breaks ) )
    for( i in 1:length(log.breaks) ) {      

      ##dbh <- x[s,]$DBH / 10.0
      ##tht <- x[s,]$THT

      ## do not convert. 
      dbh <- x[s,]$dbh 
      tht <- x[s,]$tht

      if( log.breaks[i] <= bcmof.diDBH( dbh, tht, cr, 0.0 ) ) {    
        mh <- uniroot( merch.height.func,
                      c(0,tht),
                      dbh=dbh,
                      tht=tht,
                      cr=0.60,
                      md=log.breaks[i])
        ##print( mh )
        mh.bks[i] <- mh$root
      } else {
        mh.bks[i] <- 0.0
      }
    }
    
    ##print( mh.bks )  

    ## create the final bucked stem
    ## with a stump height as the last entry
    vol.bks <- data.frame( log.grades, log.breaks, mh.bks )
    vol.bks.temp <- vol.bks[mh.bks > 0.3,] ## truncate to only those that have non-zero values
    dib <- bcmof.diDBH(dbh,tht,0.60,0.30 ) ## 1/3 feet stump height
    last.grade <- log.grades[nrow(vol.bks.temp)+1]
    last.grade.df <- data.frame(log.grades=last.grade,log.breaks=dib,mh.bks=0.30)
    
    ## prepare the "top" half of the grade data frame
    vol.bks <- rbind( vol.bks.temp, last.grade.df )
    
    ## now append those sorts that are zero
    vol.bks.null <- data.frame( log.grades, log.breaks, mh.bks )
    n.sorts <- nrow( vol.bks )
    diff.rows <- nrow( vol.bks.null ) - nrow( vol.bks )
    vol.bks <- rbind( vol.bks, tail( vol.bks.null, diff.rows ) )
    
    sed <- c(0,vol.bks[1:(n.sorts-1),]$log.breaks,rep(0,diff.rows))
    led <- c(vol.bks[1:n.sorts,]$log.breaks,rep(0,diff.rows))
    vol.bks <- cbind( vol.bks, sed, led )
    
    ## compute the smalian volumes for the 
    log.lens <- c( tht - vol.bks[1,]$mh.bks,
                  abs( diff( vol.bks$mh.bks ) )[1:n.sorts-1],
                  rep(0,diff.rows) )
    vol.bks <- cbind( vol.bks, log.lens )
    
    ##len <- abs( c( diff( vol.bks$mh.bks ) ) )
    ##vol.bks$sm.vol <- smal.vol( vol.bks$sed*0.01, vol.bks$led*0.01, vol.bks$log.lens )
    vol.bks$sm.vol <- smal.vol( vol.bks$sed, vol.bks$led, vol.bks$log.lens )
    rownames(vol.bks) <- 1:nrow(vol.bks)
    

    if( display.stems ) {
      ##print( "display.stems..." )
      hi <- 0:tht
      dbh <- rep(dbh,length(hi))
      tht <- rep(tht,length(hi))
      cr <- rep(0.6,length(hi))
      stem.tpr <- data.frame( cbind( dbh, tht, cr, hi ) )
      stem.tpr$dib <- NA
      for( i in 1:nrow(stem.tpr) ){  
        stem.tpr[i,]$dib <- bcmof.diDBH( 
                                        stem.tpr[i,]$dbh, 
                                        stem.tpr[i,]$tht, 
                                        stem.tpr[i,]$cr, 
                                        stem.tpr[i,]$hi )
        
      }
      
      ## plot the stem profile
      plot( stem.tpr$dib ~ stem.tpr$hi, type="l" )
      abline( v=vol.bks$mh.bks, lty=2, lwd=2 )
      abline( h=vol.bks$log.breaks, lty=3 )
      abline( h=0 ) 
      text( x=vol.bks$mh.bks + 1.5,
           y=vol.bks$log.breaks+1,
           labels=paste( vol.bks$log.breaks, "@", round( vol.bks$mh.bks, 2 ) ) )
    }    
      
        ## now, assign the volumes for each of the grades (sorts) to the temp data frame
    grade.vols[s,] <- vol.bks$sm.vol
  }
  
  ## add the grade.volumes to the stems (n1 - for now)
  ## you can't use merge because the initial data managers were sloppy
  ##grade.vols$CRUISEID <- n1$CRUISEID
  vol <- rowSums( grade.vols )
  x <- cbind( x, vol, grade.vols )
  
  x
}


################################################################################
## this is the new function for the species summaries.
## use the existing rconifers function definition.
## these grades are a little large and should be scaled down to include pulp 2-5"
sp.sums.2 <- function( x,
                      log.breaks=c(2,5,12,18,32,999),
                      log.grades=c("pulp","s4","s3","s2","s1","peeler") ) {

  if( class( x ) != "sample.data" ) {
    stop( "Error: x is not a sample.data object." )
    return
  }

  ## this will merchandise the plant list (sample.data$plants)
  ## and return a data.frame that contains a set of metrics
  x$plants <- generate.log.vols( x$plants, 
                                     log.breaks, 
                                     log.grades,
                                     display.stems=FALSE )
  
  x.by.sp <- split( x$plants, f=list(x$plants$sp.code ) )
  
  ## this local function computes the summaries each species.
  sp.sums.f <- function( x, npts ) {
    expf <- sum( x$expf * x$n.stems ) / npts
    tht <- sum( x$expf * x$n.stems * x$tht ) / sum( x$expf * x$n.stems )
    ba <- sum( x$expf * x$n.stems * x$dbh^2*0.0054541539 ) / npts
    expgtbh<-sum(x$n.stems[x$tht>4.5]*x$expf[x$tht>4.5])/npts
    qmd <- sqrt( ba / expgtbh / 0.0054541359 )

    ## add the number of trees per acre >= 7.0 inches
    tpa.7.plus <- sum(x$n.stems[x$dbh>=7]*x$expf[x$dbh>=7.0])/npts

    ## add the volume calculation here
    w <- as.matrix( x[,log.grades]  )
    x <- as.matrix( x$expf * x$n.stems )
    v <- t(x) %*% w / npts
    v.p <- v / sum( v )

    ## todo: add the total volume 
    total.vol <- sum(v)
    
    sp.sums <- c(qmd,tht,ba,expf,tpa.7.plus,total.vol,v,v.p)
  }
  
  df <- as.data.frame( t(sapply( x.by.sp, sp.sums.f, nrow(x$plots) )) )
  names( df ) <- c("qmd","tht","ba","expf","tpa.7.plus","sm.vol",
                   log.grades,paste( rep("p",length(log.grades)), log.grades, sep="." ))

  df
}

# $Id: interval_test.R 169 2006-01-05 05:02:44Z andrewr $

rm(list=ls())

plots <- read.csv("../data/fia_plots.csv")

plots$forest <- factor(plots$forest)

reps <- 10

ls()

## The following code extracts the plot-level estimates from the FIA database.

#load("/home/andrewr/0.rosetta/history/1.Research/Equivalence/Data/fia.wo.VSEP16-03.RData")

#dim(fia.wo)

#names(fia.wo)

#fia.wo[1:10,]

#plot(fia.wo$DBH[1:100], fia.wo$BA[1:100])
#plot(fia.wo$DBH[1:100], fia.wo$BA[1:100]*fia.wo$TF[1:100])

#plots <- aggregate(x=list(ba=fia.wo$sba),
#                   by=list(forest=fia.wo$FORCODE,
#                     plot=fia.wo$PLOT),
#                   FUN=mean)

#table(plots$forest)

#save(plots, file="../data/plots.RData")

means <- tapply(plots$ba, plots$forest, mean)

require(boot)
require(survey)

boot.mean <- function(x, index)
  c(mean(x[index]), (length(x)-1)*var(x[index])/length(x)^2)

within <- function(conf, x)
  (x >= conf[length(conf)-1] & x < conf[length(conf)])
int.above <- function(conf, x)
  (x >= conf[length(conf)])
int.below <- function(conf, x)
  (x < conf[length(conf)-1])
int.length <- function(conf)
  conf[length(conf)] - conf[length(conf)-1]

## Goal: variance stabilization.  Square root might work.

opar <- par(mfrow=c(9,2), mar=c(5,5,0,0))
for (i in 1:length(levels(plots$forest))) {
  boot.object <- boot(plots$ba[plots$forest==levels(plots$forest)[i]],
                      boot.mean, R=1999)
  scatter.smooth(boot.object$t[,1], sqrt(boot.object$t[,2]), main="",
       ylab="Standard Error",
       xlab=paste("Bootstrap Mean, Forest:", levels(plots$forest)[i]))
  boot.object <- boot(sqrt(plots$ba[plots$forest==levels(plots$forest)[i]]),
                      boot.mean, R=1999)
  scatter.smooth(boot.object$t[,1], sqrt(boot.object$t[,2]), main="",
       ylab="Standard Error",
       xlab=paste("Bootstrap Mean, Forest:", levels(plots$forest)[i]))
}
par(opar)

## Sample with replacement 

n.s <- c(16)
store <- 0

total.reps <- reps * length(n.s) * length(levels(plots$forest))

interval.lengths <- list(rep(NA, total.reps))
interval.coverage <- list(rep(NA, total.reps))
interval.above <- list(rep(NA, total.reps))
interval.below <- list(rep(NA, total.reps))

for (j in 1:length(unique(n.s))) {            # j <- 1
  n <- n.s[j]
  for (i in 1:length(levels(plots$forest))) { # i <- 1
    for (b in 1:reps) {                       # b <- 1
      store <- store + 1
      the.sample <-
        sample(plots$ba[plots$forest==levels(plots$forest)[i]],
               size=n,
               replace=TRUE)
      boot.object <- boot(the.sample, boot.mean, R=1999)
      ci <- boot.ci(boot.object)
      log.ci <- boot.ci(boot.object, type=c("norm","basic","stud"),
                        h=log, hdot=function(x) 1/x, hinv=exp)
      sqrt.ci <- boot.ci(boot.object, type=c("norm","basic","stud"),
                        h=sqrt, hdot=function(x) 2*x, hinv=function(x) x^2)
      classical <- mean(the.sample) + c(-1, 1) *
        sd(the.sample) / sqrt(n) * qt(p=0.975, df=n-1)
      interval.coverage[[store]] <-
        c(within(classical, means[i]),
          as.numeric(lapply(ci[4:8], within, x=means[i])),
          as.numeric(lapply(log.ci[4:6], within, x=means[i])),
          as.numeric(lapply(sqrt.ci[4:6], within, x=means[i])))
      interval.lengths[[store]] <- 
        c(int.length(classical),
          as.numeric(lapply(ci[4:8], int.length)),
          as.numeric(lapply(log.ci[4:6], int.length)),
          as.numeric(lapply(sqrt.ci[4:6], int.length)))
      interval.above[[store]] <- 
        c(int.above(classical, means[i]),
          as.numeric(lapply(ci[4:8], int.above, x=means[i])),
          as.numeric(lapply(log.ci[4:6], int.above, x=means[i])),
          as.numeric(lapply(sqrt.ci[4:6], int.above, x=means[i])))
      interval.below[[store]] <- 
        c(int.below(classical, means[i]),
          as.numeric(lapply(ci[4:8], int.below, x=means[i])),
          as.numeric(lapply(log.ci[4:6], int.below, x=means[i])),
          as.numeric(lapply(sqrt.ci[4:6], int.below, x=means[i])))
    }
    cat("\nFinished simulation for Forest", levels(plots$forest)[i],
        "for n =", n, "(", reps, "reps )")
  }
  cat("\n")
}

output <- 
  expand.grid(interval=c("classical",
                "normal", "basic", "studentized", "percentile", "bca",
                "log.normal", "log.basic", "log.studentized",
                "sqrt.normal", "sqrt.basic", "sqrt.studentized"),
              replicate=1:reps,
              forest = levels(plots$forest),
              n = n.s)

output$length <- unlist(interval.lengths)
output$above <- unlist(interval.above) 
output$coverage <- unlist(interval.coverage) 
output$below <- unlist(interval.below) 

results <-
  aggregate(x=list(length=output$length,
              low=output$above,
              contained=output$coverage,
              high=output$below),
            by=list(interval=output$interval,
              forest=output$forest,
              n=output$n),
            FUN=mean)

results$scaled <- results$length /
  qt(results$contained, df=as.numeric(as.character(results$n))-1) *
  qt(0.95, df=as.numeric(as.character(results$n))-1) 

results$overCover <- format((qt(0.95,
                                df=as.numeric(as.character(results$n))-1) /
                             qt(results$contained, 
                                df=as.numeric(as.character(results$n))-1))^2,
                            dig=3)

results$length <- format(results$length, dig=3)
results$low <- format(results$low, dig=3)
results$contained <- format(results$contained, dig=3)
results$high <- format(results$high, dig=3)
results$scaled <- format(results$scaled, dig=3)

results[1:12,]

overall.results <-
  aggregate(x=list(length=output$length,
              low=output$above,
              contained=output$coverage,
              high=output$below),
            by=list(interval=output$interval,
              n=output$n),
            FUN=mean)

overall.results$scaled <- overall.results$length /
  qt(overall.results$contained,
     df=as.numeric(as.character(overall.results$n))-1) *
  qt(0.95, df=as.numeric(as.character(overall.results$n))-1) 

overall.results$overCover <-
  format((qt(0.95,
             df=as.numeric(as.character(overall.results$n))-1) /
          qt(overall.results$contained, 
             df=as.numeric(as.character(overall.results$n))-1))^2,
         dig=3)

overall.results$length <- format(overall.results$length, dig=3)
overall.results$low <- format(overall.results$low, dig=3)
overall.results$contained <- format(overall.results$contained, dig=3)
overall.results$high <- format(overall.results$high, dig=3)
overall.results$scaled <- format(overall.results$scaled, dig=3)

overall.results

save.image("booted_srs.RData")

shell("./closure")

### R code from vignette source 'simulations.rnw'

###################################################
### code chunk number 1: Setup
###################################################

rm(list=ls())

options(width=65)

if(!require(lattice, quietly=TRUE)) install.packages("lattice")
if(!require(Hmisc, quietly=TRUE)) install.packages("Hmisc")
if(!require(rconifers, quietly=TRUE)) install.packages("rconifers")

## this is for
b201.age <- 2:15*10
b201.tht <- c(47,83,108,125,140,152,163,172,180,187,192,196,199,201)
b201.tpa <- c(756,483,335,248,195,160,136,118,106,95,87,80,75,71)
b201.qmd <- c(4.9,7.6,10.2,12.8,15.2,17.5,19.6,21.4,23.1,24.6,26.1,27.5,28.8,30)
b201.ba <- c(99,152,191,220,244,264,280,294,306,317,326,335,343,351)
b201.vol <- c(1650,4330,6900,9320,11450,13300,14990,16400,17550,18510,19320,20000,20640,21270)
b201.mai <- c(77,136,164,177,181,181,178,173) ## only goes from 20 to 90 age[1:8]



###################################################
### code chunk number 2: simulations.rnw:213-214 (eval = FALSE)
###################################################
## install.packages("rconifers")


###################################################
### code chunk number 3: simulations.rnw:220-224
###################################################

library(rconifers)
set.species.map(set.variant(1))



###################################################
### code chunk number 4: Maybe should be ... (eval = FALSE)
###################################################
## set.variant(1)
## data( smc )
## set.species.map( smc )


###################################################
### code chunk number 5: simulations.rnw:238-241
###################################################
dim(smc)
names(smc)
head(smc)


###################################################
### code chunk number 6: simulations.rnw:281-285
###################################################

data(plots.smc)
data(plants.smc)



###################################################
### code chunk number 7: simulations.rnw:291-298
###################################################

sample.3 <- list(plots = plots.smc, 
                 plants = plants.smc, 
                 age = 3, 
                 x0 = 0.0)
class(sample.3)  <- "sample.data"



###################################################
### code chunk number 8: simulations.rnw:304-307
###################################################

summary(sample.3)



###################################################
### code chunk number 9: simulations.rnw:325-333
###################################################
summary(sample.3)
sample.25 <- project(sample.3, 
                     22,
                     control = list(rand.err = 0,
                                    rand.seed = 0,
                                    endemic.mort = 0,
                                    sdi.mort = 0))
summary(sample.25)


###################################################
### code chunk number 10: simulations.rnw:371-391
###################################################

res.v <- vector(length = 98, mode = "list")
s0 <- sample.3
res.v[[1]] <- data.frame(age = s0$age, sp.sums(s0)["DF",])

## grow from age 3 to age 100 (97 more years)
for(m in 1:97) {
  ## project s0 in one year intervals
  s1 <- project(s0, 1, control = list(rand.err = 0,
                                      rand.seed = 0,
                                      endemic.mort = 1,
                                      sdi.mort = 1))          
  res.v[[m+1]] <- data.frame(age = s1$age, 
                             sp.sums(s1)["DF",])
  s0 <- s1      
}

res.v <- do.call(rbind, res.v)
res <- res.v



###################################################
### code chunk number 11: fig-rconifers
###################################################

opar <- par(mfcol=c(1,2), las=1, cex=0.8)

################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
#plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="n", 
     ylim=c(0,350),
     xlim=c(0,100),
##      main="Trees per Acre > 7 Inches",
##      ylab="Trees/Acre > 7.0 Inches", 
      main="Trees per Acre (All Stems)",
      ylab="Trees/Acre (All Stems)", 
     xlab="Total Age, in years", col="black")
grid(lty=1)
par(new=TRUE)
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
##plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="l", 
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
par(new=TRUE)
##plot(rcon.531$tpa.7.plus ~ rcon.531$age, 
##plot(rcon.531$expf ~ rcon.531$age, 
plot(res$expf ~ res$age, 
     type="l", 
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
lines(b201.age, b201.tpa, lty=2, col="gray")


################################################################################
## plot the basal area 
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="n", 
     ylim=c(0,300),
     xlim=c(0,100),
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years", col="black")
grid(lty=1)
par(new=TRUE)
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years", col="black")
par(new=TRUE)
##plot(rcon.531$ba ~ rcon.531$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
lines(b201.age, b201.ba, lty=2, col="gray")



###################################################
### code chunk number 12: fig-rconifers
###################################################

opar <- par(mfcol=c(1,2), las=1, cex=0.8)

################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
#plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="n", 
     ylim=c(0,350),
     xlim=c(0,100),
##      main="Trees per Acre > 7 Inches",
##      ylab="Trees/Acre > 7.0 Inches", 
      main="Trees per Acre (All Stems)",
      ylab="Trees/Acre (All Stems)", 
     xlab="Total Age, in years", col="black")
grid(lty=1)
par(new=TRUE)
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
##plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="l", 
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
par(new=TRUE)
##plot(rcon.531$tpa.7.plus ~ rcon.531$age, 
##plot(rcon.531$expf ~ rcon.531$age, 
plot(res$expf ~ res$age, 
     type="l", 
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
lines(b201.age, b201.tpa, lty=2, col="gray")


################################################################################
## plot the basal area 
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="n", 
     ylim=c(0,300),
     xlim=c(0,100),
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years", col="black")
grid(lty=1)
par(new=TRUE)
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years", col="black")
par(new=TRUE)
##plot(rcon.531$ba ~ rcon.531$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA, col="black")
lines(b201.age, b201.ba, lty=2, col="gray")



###################################################
### code chunk number 13: shlib
###################################################

system("R CMD SHLIB chambers-1980.c")



###################################################
### code chunk number 14: simulations.rnw:703-706
###################################################

dyn.load("chambers-1980.so")



###################################################
### code chunk number 15: simulations.rnw:727-736
###################################################

site.index <- 120.0
total.age <- 60

.C("chambers_1980_stand_mean_height",
              as.double(site.index),
              as.double(total.age),
              smh = as.double(0))$smh



###################################################
### code chunk number 16: simulations.rnw:748-757
###################################################

chambers.1980.smh <- function(site.index,total.age) {
  ret.val <- .C("chambers_1980_stand_mean_height",
                as.double(site.index),
                as.double(total.age),
                smh = as.double(0))$smh
  ret.val
}



###################################################
### code chunk number 17: simulations.rnw:766-769
###################################################

chambers.1980.smh(120.0, 60.0)



###################################################
### code chunk number 18: simulations.rnw:782-784
###################################################
source("chambers-1980.r")



###################################################
### code chunk number 19: simulations.rnw:802-821
###################################################

chambers.1980 <- function(ages = 1:100, 
                          site = 125.0, 
                          pnba = 1.0) {
  ret.val <- matrix(0, length(ages), 5)
  for(i in ages) {    
    res <- c(i, 
             chambers.1980.adbh(site, i, pnba), ## Table 5
             chambers.1980.smh(site, i),        ## Table 16
             chambers.1980.nba(site, i),        ## Table 1
             chambers.1980.ntpa(site, i, pnba)  ## Table 3
            )
    ret.val[i,] <- res
  }
  ret.val <- as.data.frame(ret.val)
  names(ret.val) <- c("age","qmd","tht","ba","expf")
  ret.val
}



###################################################
### code chunk number 20: fig-chambers-1980
###################################################

## you need to convert between kings and jims

ch.site <- 125.0
pnba <- 1.0
ch.111 <- chambers.1980(ages=1:100, site=ch.site, pnba=pnba)
ch.531 <- chambers.1980(ages=1:100, site=ch.site, pnba=pnba)


## this code is not presented
opar <- par(mfcol=c(1,2), las=1, cex=0.8)
##opar <- par(mfcol=c(2,2), las=1, cex=0.8)


################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
plot(ch.111$expf ~ ch.111$age, 
     type="n", 
     ylim=c(0,350),
     xlim=c(0,100),
     main="Trees per Acre",
     ylab="Trees/Acre > 7.0 Inches", 
     xlab="Total Age, in years")
grid(lty=1)
par(new=TRUE)
plot(ch.111$expf ~ ch.111$age, 
     type="l", col="darkgray",
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## plot(ch.531$expf ~ ch.531$age, 
##      type="l", col="darkgray",
##      ylim=c(0,350),
##      xlim=c(0,100),
##      main="Trees per Acre",
##      ylab="Trees/Acre > 7.0 Inches", 
##      xlab="Total Age, in years")
################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
par(new=TRUE)
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
##plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="l", col="black",
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## ##plot(rcon.531$tpa.7.plus ~ rcon.531$age, 
## plot(rcon.531$expf ~ rcon.531$age, 
##      type="l", col="black",
##      ylim=c(0,350),
##      xlim=c(0,100),
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
lines(b201.age, b201.tpa, lty=2, col="darkgray")

################################################################################
## plot the basal area for the two models
################################################################################
plot(ch.111$ba ~ ch.111$age, 
     type="n", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="black",
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years")
grid(lty=1)
par(new=TRUE)
plot(ch.111$ba ~ ch.111$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="darkgray",
     main=NA,
     ylab=NA, 
     xlab=NA)
par(new=TRUE)
## plot(ch.531$ba ~ ch.531$age, 
##      type="l", 
##      ylim=c(0,300),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="black",
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## plot(rcon.531$ba ~ rcon.531$age, 
##      type="l", 
##      ylim=c(0,300),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
lines(b201.age, b201.ba, lty=2, col="darkgray")


################################################################################
## plot the quadratic mean diameter for the two models
################################################################################
## plot(ch.111$qmd ~ ch.111$age, 
##      type="n", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main="Mean Stem Diameter",
##      ylab="Inches", 
##      xlab="Total Age, in years")
## grid(lty=1)
## par(new=TRUE)
## plot(ch.111$qmd ~ ch.111$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(ch.531$qmd ~ ch.531$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(rcon.111$qmd ~ rcon.111$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(rcon.531$qmd ~ rcon.531$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)



###################################################
### code chunk number 21: fig-chambers-1980
###################################################

## you need to convert between kings and jims

ch.site <- 125.0
pnba <- 1.0
ch.111 <- chambers.1980(ages=1:100, site=ch.site, pnba=pnba)
ch.531 <- chambers.1980(ages=1:100, site=ch.site, pnba=pnba)


## this code is not presented
opar <- par(mfcol=c(1,2), las=1, cex=0.8)
##opar <- par(mfcol=c(2,2), las=1, cex=0.8)


################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
plot(ch.111$expf ~ ch.111$age, 
     type="n", 
     ylim=c(0,350),
     xlim=c(0,100),
     main="Trees per Acre",
     ylab="Trees/Acre > 7.0 Inches", 
     xlab="Total Age, in years")
grid(lty=1)
par(new=TRUE)
plot(ch.111$expf ~ ch.111$age, 
     type="l", col="darkgray",
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## plot(ch.531$expf ~ ch.531$age, 
##      type="l", col="darkgray",
##      ylim=c(0,350),
##      xlim=c(0,100),
##      main="Trees per Acre",
##      ylab="Trees/Acre > 7.0 Inches", 
##      xlab="Total Age, in years")
################################################################################
## plot the trees per acre, 7.0 inches dbh and larger
## table three (3). 
## reflects pure even-aged second growth stand
par(new=TRUE)
##plot(rcon.111$tpa.7.plus ~ rcon.111$age, 
##plot(rcon.111$expf ~ rcon.111$age, 
plot(res$expf ~ res$age, 
     type="l", col="black",
     ylim=c(0,350),
     xlim=c(0,100),
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## ##plot(rcon.531$tpa.7.plus ~ rcon.531$age, 
## plot(rcon.531$expf ~ rcon.531$age, 
##      type="l", col="black",
##      ylim=c(0,350),
##      xlim=c(0,100),
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
lines(b201.age, b201.tpa, lty=2, col="darkgray")

################################################################################
## plot the basal area for the two models
################################################################################
plot(ch.111$ba ~ ch.111$age, 
     type="n", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="black",
     main="Basal Area",
     ylab="Ft^2/Acre", 
     xlab="Total Age, in years")
grid(lty=1)
par(new=TRUE)
plot(ch.111$ba ~ ch.111$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="darkgray",
     main=NA,
     ylab=NA, 
     xlab=NA)
par(new=TRUE)
## plot(ch.531$ba ~ ch.531$age, 
##      type="l", 
##      ylim=c(0,300),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
##plot(rcon.111$ba ~ rcon.111$age, 
plot(res$ba ~ res$age, 
     type="l", 
     ylim=c(0,300),
     xlim=c(0,100),
     col="black",
     main=NA,
     ylab=NA, 
     xlab=NA)
## par(new=TRUE)
## plot(rcon.531$ba ~ rcon.531$age, 
##      type="l", 
##      ylim=c(0,300),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
lines(b201.age, b201.ba, lty=2, col="darkgray")


################################################################################
## plot the quadratic mean diameter for the two models
################################################################################
## plot(ch.111$qmd ~ ch.111$age, 
##      type="n", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main="Mean Stem Diameter",
##      ylab="Inches", 
##      xlab="Total Age, in years")
## grid(lty=1)
## par(new=TRUE)
## plot(ch.111$qmd ~ ch.111$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(ch.531$qmd ~ ch.531$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="darkgray",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(rcon.111$qmd ~ rcon.111$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)
## par(new=TRUE)
## plot(rcon.531$qmd ~ rcon.531$age, 
##      type="l", 
##      ylim=c(0,25),
##      xlim=c(0,100),
##      col="black",
##      main=NA,
##      ylab=NA, 
##      xlab=NA)



###################################################
### code chunk number 22: simulations.rnw:1099-1114
###################################################
bcmof.diDBH <- function(dbh, tht, cr, hi) {
  p <- 0.25    
  dib <- 1.02453 * dbh^0.88809 * 1.00035^dbh
  X <- (1.0 - sqrt(hi / tht)) / (1.0 - sqrt(p))
  Z = hi / tht
  a = 0.95086 * Z * Z;
  b = -0.18090 * log(Z + 0.001);
  c = 0.61407 * sqrt(Z) +  -0.35105 * exp(Z);
  d = 0.05686 * (dbh / tht); 
  retval <- (dib * X^(a + b + c + d))
  retval[tht < hi] <- 0
  retval
}




###################################################
### code chunk number 23: simulations.rnw:1133-1140
###################################################

merch.height.func <- function(hi, dbh, tht, cr, md) {
  dib <- bcmof.diDBH(dbh, tht, cr, hi)
  diff <- dib - md 
  diff
}



###################################################
### code chunk number 24: simulations.rnw:1150-1160
###################################################

mh <- uniroot(merch.height.func,
                c(0, 27),
                dbh = 45.0,
                tht = 27,
                cr = 0.60,
                md = 10.0)

mh



###################################################
### code chunk number 25: simulations.rnw:1187-1191
###################################################

min.dib.check <- bcmof.diDBH(45, 27, 0.60, mh$root)
min.dib.check



###################################################
### code chunk number 26: simulations.rnw:1227-1243
###################################################

smal.vol <- function(d1, d2, l) {
  c <- 0.0001570796 ## in m^3
  smal.vol <- (0.25 * d1^2 + 0.25 * d2^2) * l * c
  smal.vol
}


log.breaks=c(2,5,12,18,32,999)
log.grades=c("pulp","s4","s3","s2","s1","peeler")
grade.names <- c("Pulp","#4 Sawlog","#3 Sawlog",
                 "#2 Sawlog","#1 Sawlog", "Peeler")

## log.breaks <- c(5,15,30,75,999)
## log.grades <- c("pulp","s3","s2","s1","peeler")



###################################################
### code chunk number 27: log loops
###################################################

mh.bks <- rep(0, length(log.breaks))
for(i in 1:length(log.breaks)) {      
  dbh <- 45.0; tht <- 27; cr <- 0.60          
  if(log.breaks[i] <= bcmof.diDBH(dbh, tht, cr, 0.0)) {    
    mh <- uniroot(merch.height.func,
                  c(0, tht),
                  dbh = dbh,
                  tht = tht,
                  cr = 0.60,
                  md = log.breaks[i])
    mh.bks[i] <- mh$root
  } else {
    mh.bks[i] <- 0.0
  }
}



###################################################
### code chunk number 28: simulations.rnw:1296-1300
###################################################

log.breaks
mh.bks



###################################################
### code chunk number 29: simulations.rnw:1326-1329
###################################################

source("../../scripts/generate_log_volumes.r")



###################################################
### code chunk number 30: fig-log-bucking
###################################################

hi <- 0:tht
dbh <- rep(45.0,length(hi))
tht <- rep(27,length(hi))
cr <- rep(0.6,length(hi))
stem.tpr <- data.frame(cbind(dbh, tht, cr, hi))
stem.tpr$dib <- NA

#for(i in 1:nrow(stem.tpr)){  
#  stem.tpr[i,]$dib <- bcmof.diDBH(
#                                  stem.tpr[i,]$dbh, 
#                                  stem.tpr[i,]$tht, 
#                                  stem.tpr[i,]$cr, 
#                                  stem.tpr[i,]$hi)
#  
#}

## Use vectorized function!

stem.tpr$dib <- with(stem.tpr, bcmof.diDBH(dbh, tht, cr, hi))  

## plot the stem profile
opar <- par(las=1, cex=0.8)
plot(stem.tpr$dib ~ stem.tpr$hi, type="l", 
     ylab=expression(dib),
     xlab=expression(h[i]),
     main="Example Stem with Crosscuts", xlim=c(-1,35))
abline(v=mh.bks, lty=2, lwd=2)
##abline(v=0.3, lty=2, lwd=2)
abline(h=log.breaks, lty=3)
abline(h=0) 
text(x=mh.bks + 4.5,
     y=log.breaks + 2.5,
     labels=paste(log.breaks, " cm @", round(mh.bks, 2), " m"))
par(opar)



###################################################
### code chunk number 31: fig-log-bucking
###################################################

hi <- 0:tht
dbh <- rep(45.0,length(hi))
tht <- rep(27,length(hi))
cr <- rep(0.6,length(hi))
stem.tpr <- data.frame(cbind(dbh, tht, cr, hi))
stem.tpr$dib <- NA

#for(i in 1:nrow(stem.tpr)){  
#  stem.tpr[i,]$dib <- bcmof.diDBH(
#                                  stem.tpr[i,]$dbh, 
#                                  stem.tpr[i,]$tht, 
#                                  stem.tpr[i,]$cr, 
#                                  stem.tpr[i,]$hi)
#  
#}

## Use vectorized function!

stem.tpr$dib <- with(stem.tpr, bcmof.diDBH(dbh, tht, cr, hi))  

## plot the stem profile
opar <- par(las=1, cex=0.8)
plot(stem.tpr$dib ~ stem.tpr$hi, type="l", 
     ylab=expression(dib),
     xlab=expression(h[i]),
     main="Example Stem with Crosscuts", xlim=c(-1,35))
abline(v=mh.bks, lty=2, lwd=2)
##abline(v=0.3, lty=2, lwd=2)
abline(h=log.breaks, lty=3)
abline(h=0) 
text(x=mh.bks + 4.5,
     y=log.breaks + 2.5,
     labels=paste(log.breaks, " cm @", round(mh.bks, 2), " m"))
par(opar)



###################################################
### code chunk number 32: simulations.rnw:1416-1421
###################################################

## the log.breaks are in cm
sp2 <- sp.sums.2(sample.25, log.breaks/2.54, log.grades)
sp2[,c("sm.vol", log.grades)]



###################################################
### code chunk number 33: summary
###################################################

ch <- chambers.1980(ages = 1:200, site = 120, pnba = 1.0)      
ch.stem <- data.frame(plot = 1,
                         sp.code = "DF", 
                         d6 = NA,
                         dbh = ch[50,]$qmd, 
                         tht = ch[50,]$tht,
                         cr = 0.6,
                         n.stems = 1,
                         expf = ch[50,]$expf,
                         crown.width = NA,
                         errors = 0)
ch.plot <- data.frame(plot = 1,
                      elevation = NA,
                      slope = NA,
                      aspect = NA,
                      whc = NA,
                      map = NA,
                      si30 = 120)

ch.50 <- list(plots = ch.plot, plants = ch.stem, age = 50)
class(ch.50) <- "sample.data"

sh2 <- sp.sums.2(ch.50, log.breaks, log.grades)
sh2[,c("qmd","ba","expf","sm.vol",log.grades)]



###################################################
### code chunk number 34: simulations.rnw:1511-1514
###################################################
dia.obs <- rnorm(n = round(ch[50,]$expf)*10, 
                 mean = ch[50,]$qmd,
                 sd = 0.20*ch[50,]$qmd)


###################################################
### code chunk number 35: generate
###################################################

## generate the plant records
plant.obs <- data.frame(plot = 1,
                        sp.code = "DF",
                        d6 = NA,
                        dbh = dia.obs,
                        tht = 4.5 + exp(7.262195456 + 
                              -5.899759104 * 
                              dia.obs^-0.287207389), 
                        cr = 0.40,
                        n.stems = 1,
                        expf = 1/10,
                        crown.width = NA)

plot.obs <- data.frame(plot = 1,
                       elevation = 1000,
                       slope = 0,
                       aspect = 0,
                       whc = NA,
                       map = NA,
                       si30 = 85.0)

ch.50 <- list(plots = plot.obs, 
              plants = plant.obs, 
              age = ch[50,]$age,
              x0 = NA)
class(ch.50)  <- "sample.data"



###################################################
### code chunk number 36: m.o.i.
###################################################

sh2 <- sp.sums.2(ch.50, log.breaks, log.grades)
sh2[, c("qmd", "ba", "expf", "sm.vol", log.grades)]



###################################################
### code chunk number 37: simulations.rnw:1581-1584 (eval = FALSE)
###################################################
## 
## dyn.unload("chambers-1980.so")
## 


###################################################
### code chunk number 38: simulations.rnw:1607-1646
###################################################

## this is where the results get loaded from the earlier version.
load("res")

rcon.res <- res$rconifers
rcon <- rcon.res[rcon.res$veg.ctrl == 1.0, ]
a <- rcon[1:169,]

## with the chambers simulation for the same run
chambers.res <- res$chambers
rch <- chambers.res[chambers.res$veg.ctrl == 1.0,]
b <- rch[1:169,]

## load b.prime here
load(file="b.prime")

b.prime$mai <- b.prime$sm.vol / b.prime$age
## hack to extend b.prime to the starting age.
b.prime <-  rbind(b[1:10,], b.prime[2:nrow(b.prime),])

## these will be a little off. 
## so you can't create a vector with 100 years.
## you have to go from age ? to age ?
alpha <- rep(1, length(a$age))
#a$age < 20

## alpha, now a T x 1 vector of weights
## where the attributes for the two data.frames
## are combined into the 
ds <- 30
de <- 40
alpha[a$age < ds] <- 1
alpha[a$age >= ds & a$age <= de] <- seq(1, 0, length.out=length(ds:de))
alpha[a$age > de] <- 0

c <- alpha * a + (1 - alpha) * b
c.prime <- alpha * a + (1 - alpha) * b.prime




###################################################
### code chunk number 39: fig-mega-plot
###################################################

##opar <- par(mfrow=c(3,2), las=1, cex=0.8)
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,4.1,2.1) )

## order is bottom, west, north, east
## $mar
## [1] 5.1 4.1 4.1 2.1
opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,3.1,2.1) )

################################################################################
## do this for the tpa... 
plot(a$expf ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,350), lty=1,
     main="Stems per Acre",
     xlab="Age, in Years",
     ylab="Stems per Acre")
grid(lty=1)
par(new=TRUE)
plot(a$expf ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
##lines(c$age, c$tpa.7.plus, lwd=5, col="gray")
lines(c$age, c$expf, lwd=5, col="gray")
par(new=TRUE)
##plot(rcon$tpa.7.plus ~ rcon$age, 
plot(a$expf ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$expf ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.tpa, col="gray", lty=2, lwd=2)

################################################################################
## do this for the basal area... 
##plot(rcon$ba ~ rcon$age, 
plot(a$ba ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
     main="Stand Basal Area",
     xlab="Age, in Years",
     ylab="Stand Basal Area, in Feet^2")
grid(lty=1)
par(new=TRUE)
plot(a$ba ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$ba, lwd=5, col="gray")
par(new=TRUE)
plot(a$ba ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$ba ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.ba, col="gray", lty=2, lwd=2)
##lines(b210.age, b210.ba, col="gray", lty=2)
##lines(b210.age, b210.tpa, col="gray", lty=2)


################################################################################
## do this for the height... 
plot(a$tht ~ a$age, 
     type="n", xlim=c(0,150), ylim=c(0,250), lty=1,
     main="Mean Stand Height",
     xlab="Age, in Years",
     ylab=expression(feet))
grid(lty=1)
par(new=TRUE)
plot(a$tht ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$tht, lwd=5, col="gray")
par(new=TRUE)
plot(a$tht ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$tht ~ b$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
## peaks at 119 years.
##abline(v=119)
##lines(b201.age, b201.tht, lwd=5, col="gray")
lines(b201.age, b201.tht, col="gray", lty=2, lwd=2)
       

################################################################################
## do this for volume... 
plot(a$sm.vol ~ a$age, 
     type="n", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main="Smalian Volume",
     xlab="Age, in Years",
     ylab=expression(ft^3))
grid(lty=1)
par(new=TRUE)
plot(a$sm.vol ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
##lines(c$age, c$sm.vol, lwd=5, col="gray")
lines(c.prime$age, c.prime$sm.vol, lwd=5, col="gray")
par(new=TRUE)
plot(a$sm.vol ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$sm.vol ~ b$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.vol, col="gray", lty=2, lwd=2)
lines(b.prime$age, b.prime$sm.vol, col="black", lty=2, lwd=2)
##lines(c.prime$age, c.prime$sm.vol, lwd=5, col="gray")
##lines(b.prime$age, b.prime$sm.vol, col="black", lty=2, lwd=2)


################################################################################
## mai
################################################################################
## plot the unfixed version
plot(a$mai ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
     ##main="Mean Annual Increment\n(c=alpha a + (1-\alpha)b)",
     ##main="Mean Annual Increment\nWithout DDIST (c)",
     main="Mean Annual Increment\nfrom Stand Summaries",
     xlab="Age, in Years",
     ylab="Cubic Volume/Age")
grid(lty=1)
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$mai, lwd=5, col="gray")
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$mai ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age[1:8], b201.mai, lty=2, col="gray", lwd=2)




################################################################################
## plot the fixed version
plot(a$mai ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
##     main="Mean Annual Increment\n(c.prime=alpha a + (1-\alpha)b.prime)",
     main="Mean Annual Increment\nfrom Tree Lists",
     xlab="Age, in Years",
     ylab="Cubic Volume/Age")
grid(lty=1)
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
##lines(c$age, c$mai, lwd=5, col="gray")
## c.prime is the combined values, with the 
## ddist correction (flewelling correction)
lines(c.prime$age, c.prime$mai, lwd=5, col="gray")
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.mai ~ b201.age[1:8], lty=2, lwd=2, col="gray")
par(new=TRUE)
plot(b.prime$mai ~ b.prime$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)

par(opar)



###################################################
### code chunk number 40: fig-mega-plot
###################################################

##opar <- par(mfrow=c(3,2), las=1, cex=0.8)
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,4.1,2.1) )

## order is bottom, west, north, east
## $mar
## [1] 5.1 4.1 4.1 2.1
opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,3.1,2.1) )

################################################################################
## do this for the tpa... 
plot(a$expf ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,350), lty=1,
     main="Stems per Acre",
     xlab="Age, in Years",
     ylab="Stems per Acre")
grid(lty=1)
par(new=TRUE)
plot(a$expf ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
##lines(c$age, c$tpa.7.plus, lwd=5, col="gray")
lines(c$age, c$expf, lwd=5, col="gray")
par(new=TRUE)
##plot(rcon$tpa.7.plus ~ rcon$age, 
plot(a$expf ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$expf ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,350), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.tpa, col="gray", lty=2, lwd=2)

################################################################################
## do this for the basal area... 
##plot(rcon$ba ~ rcon$age, 
plot(a$ba ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
     main="Stand Basal Area",
     xlab="Age, in Years",
     ylab="Stand Basal Area, in Feet^2")
grid(lty=1)
par(new=TRUE)
plot(a$ba ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$ba, lwd=5, col="gray")
par(new=TRUE)
plot(a$ba ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$ba ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.ba, col="gray", lty=2, lwd=2)
##lines(b210.age, b210.ba, col="gray", lty=2)
##lines(b210.age, b210.tpa, col="gray", lty=2)


################################################################################
## do this for the height... 
plot(a$tht ~ a$age, 
     type="n", xlim=c(0,150), ylim=c(0,250), lty=1,
     main="Mean Stand Height",
     xlab="Age, in Years",
     ylab=expression(feet))
grid(lty=1)
par(new=TRUE)
plot(a$tht ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$tht, lwd=5, col="gray")
par(new=TRUE)
plot(a$tht ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$tht ~ b$age, 
     type="l", xlim=c(0,150), ylim=c(0,250), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
## peaks at 119 years.
##abline(v=119)
##lines(b201.age, b201.tht, lwd=5, col="gray")
lines(b201.age, b201.tht, col="gray", lty=2, lwd=2)
       

################################################################################
## do this for volume... 
plot(a$sm.vol ~ a$age, 
     type="n", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main="Smalian Volume",
     xlab="Age, in Years",
     ylab=expression(ft^3))
grid(lty=1)
par(new=TRUE)
plot(a$sm.vol ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
##lines(c$age, c$sm.vol, lwd=5, col="gray")
lines(c.prime$age, c.prime$sm.vol, lwd=5, col="gray")
par(new=TRUE)
plot(a$sm.vol ~ a$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$sm.vol ~ b$age, 
     type="l", xlim=c(0,150), ylim=c(0,25000), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age, b201.vol, col="gray", lty=2, lwd=2)
lines(b.prime$age, b.prime$sm.vol, col="black", lty=2, lwd=2)
##lines(c.prime$age, c.prime$sm.vol, lwd=5, col="gray")
##lines(b.prime$age, b.prime$sm.vol, col="black", lty=2, lwd=2)


################################################################################
## mai
################################################################################
## plot the unfixed version
plot(a$mai ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
     ##main="Mean Annual Increment\n(c=alpha a + (1-\alpha)b)",
     ##main="Mean Annual Increment\nWithout DDIST (c)",
     main="Mean Annual Increment\nfrom Stand Summaries",
     xlab="Age, in Years",
     ylab="Cubic Volume/Age")
grid(lty=1)
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
lines(c$age, c$mai, lwd=5, col="gray")
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
plot(b$mai ~ b$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.age[1:8], b201.mai, lty=2, col="gray", lwd=2)




################################################################################
## plot the fixed version
plot(a$mai ~ a$age, 
     type="n", xlim=c(0,100), ylim=c(0,300), lty=1,
##     main="Mean Annual Increment\n(c.prime=alpha a + (1-\alpha)b.prime)",
     main="Mean Annual Increment\nfrom Tree Lists",
     xlab="Age, in Years",
     ylab="Cubic Volume/Age")
grid(lty=1)
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
par(new=TRUE)
##lines(c$age, c$mai, lwd=5, col="gray")
## c.prime is the combined values, with the 
## ddist correction (flewelling correction)
lines(c.prime$age, c.prime$mai, lwd=5, col="gray")
par(new=TRUE)
plot(a$mai ~ a$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=1,
     main=NA,
     xlab=NA,
     ylab=NA)
lines(b201.mai ~ b201.age[1:8], lty=2, lwd=2, col="gray")
par(new=TRUE)
plot(b.prime$mai ~ b.prime$age, 
     type="l", xlim=c(0,100), ylim=c(0,300), lty=2,
     main=NA,
     xlab=NA,
     ylab=NA)

par(opar)



###################################################
### code chunk number 41: simulations.rnw:1980-1997
###################################################

a.max.mai <- a[which.max(a$mai),]
b.max.mai <- b[which.max(b$mai),]
b.prime.max.mai <- b.prime[b.prime$mai == max(b.prime$mai),]

c.max.mai <- c[c$mai == max(c$mai),]
c.max.mai$col <- "c"

c.prime.max.mai <- c.prime[c.prime$mai == max(c.prime$mai),]
c.prime.max.mai$col <- "c.prime"

max.mais <- rbind(a.max.mai, 
                  b.max.mai,
                  b.prime.max.mai,
                  c.max.mai,
                  c.prime.max.mai)



###################################################
### code chunk number 42: simulations.rnw:2007-2011
###################################################

small.mai <- max.mais[max.mais$mai == min(max.mais$mai) & !is.na(max.mais$col),]
biggest.mai <- max.mais[max.mais$mai == max(max.mais$mai) & !is.na(max.mais$col),]



###################################################
### code chunk number 43: simulations.rnw:2030-2040
###################################################

cap <- paste("Statistics for simulations at the maximum mean annual increment for each of the models.") 
basic <- c("col", "age","qmd","tht","ba","expf", "tpa.7.plus", "sm.vol")
mm2 <- max.mais[,basic]

names( mm2 ) <- c("Simulation", "Age","QMD (in)","Height (ft)","Basal Area ($\\mbox{ft}^2$)", "TPA", "TPA 7\'\'+", "Volume ($\\mbox{ft}^3$)")

latex(mm2, rowlabel=NULL, rowname=NULL, file="",caption=cap,
     label="tab:max-mais-metrics", digits=1, placement="!tp")



###################################################
### code chunk number 44: fig-merging-two-models-soln-vol-dist
###################################################

## you have to reduce the vertical space between the images.
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(0,0,0,0), oma=c(0,0,0,0) )

##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(.5,.5,.5,.5), oma=c(0,0,0,0) )

## order is bottom, west, north, east
## $mar
## [1] 5.1 4.1 4.1 2.1
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,4.1,2.1) )
opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,3.1,2.1) )


grd <- 1
ylim <- c(0,50)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

################################################################################
## plot the #2 log grade results
## plot(a[,c(log.grades[2])] ~ a[,c("age")], main=grade.names[2], type="n", lty=1, ylim=c(0,1000), xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
## grid(lty=1)
## par(new=TRUE)
## plot(c.prime[,c(log.grades[2])] ~ c.prime[,c("age")], main=NA,  lwd=5, lty=1,col="gray", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## par(new=TRUE)
## ## a is a thin solid black line
## plot(a[,c(log.grades[2])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=c(0,1000), xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
## par(new=TRUE)
## ## b is a thin solid gray line
## plot(b[,c(log.grades[2])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## ##par(new=TRUE)
## ##plot(c[,c(log.grades[2])] ~ c[,c("age")], main=NA, type="l", lty=2, col="gray", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## par(new=TRUE)
## ## c.prime is a thin dashed black line
## plot(c.prime[,c(log.grades[2])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)

grd <- 2
ylim <- c(0,1000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

################################################################################
## plot the #3 log grade results
ylim <- c(0,15000)
plot(a[,c(log.grades[3])] ~ a[,c("age")], main=grade.names[3], type="n", lty=1, ylim=ylim, xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
## c.prime is wide, gray, solid
plot(c.prime[,c(log.grades[3])] ~ c.prime[,c("age")], main=NA, type="l",lwd=5, col="gray",  lty=1, ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
par(new=TRUE)
## a is the thin, black, solid
plot(a[,c(log.grades[3])] ~ a[,c("age")], main=NA, type="l", lwd=2, col="black", lty=1, ylim=ylim, xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is thin, gray, solid
plot(b[,c(log.grades[3])] ~ b[,c("age")], main=NA, type="l", lwd=2, col="gray",lty=1,  ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
par(new=TRUE)
## c is thin, gray, dashed
#plot(c[,c(log.grades[3])] ~ c[,c("age")], main=NA, type="l", lwd=1, col="gray", lty=2,  ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
#par(new=TRUE)
## c.prime thin, black, dashed
plot(c.prime[,c(log.grades[3])] ~ c.prime[,c("age")], main=NA, type="l",  lwd=2, col="black", lty=2, ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)


## a = thin solid black
## b = thick dashed gray
## c = thin dashed gray
## c.prime = thin dashed black
  
################################################################################
## plot the #4 log grade results
grd <- 4
ylim <- c(0,15000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

grd <- 5
ylim <- c(0,5000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)


################################################################################
## plot the #6 log grade results
grd <- 6
ylim <- c(0,500)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
##par(new=TRUE)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

## c the a thin, solid, gray line
##plot(c[,c(log.grades[grd])] ~ c[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
##par(new=TRUE)

par(opar)




###################################################
### code chunk number 45: fig-merging-two-models-soln-vol-dist
###################################################

## you have to reduce the vertical space between the images.
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(0,0,0,0), oma=c(0,0,0,0) )

##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(.5,.5,.5,.5), oma=c(0,0,0,0) )

## order is bottom, west, north, east
## $mar
## [1] 5.1 4.1 4.1 2.1
##opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,4.1,2.1) )
opar <- par(mfrow=c(3,2), las=1, cex=0.8, mar=c(4.1,4.1,3.1,2.1) )


grd <- 1
ylim <- c(0,50)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

################################################################################
## plot the #2 log grade results
## plot(a[,c(log.grades[2])] ~ a[,c("age")], main=grade.names[2], type="n", lty=1, ylim=c(0,1000), xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
## grid(lty=1)
## par(new=TRUE)
## plot(c.prime[,c(log.grades[2])] ~ c.prime[,c("age")], main=NA,  lwd=5, lty=1,col="gray", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## par(new=TRUE)
## ## a is a thin solid black line
## plot(a[,c(log.grades[2])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=c(0,1000), xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
## par(new=TRUE)
## ## b is a thin solid gray line
## plot(b[,c(log.grades[2])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## ##par(new=TRUE)
## ##plot(c[,c(log.grades[2])] ~ c[,c("age")], main=NA, type="l", lty=2, col="gray", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)
## par(new=TRUE)
## ## c.prime is a thin dashed black line
## plot(c.prime[,c(log.grades[2])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=c(0,1000), xlim=c(0,100), xlab=NA, ylab=NA)

grd <- 2
ylim <- c(0,1000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

################################################################################
## plot the #3 log grade results
ylim <- c(0,15000)
plot(a[,c(log.grades[3])] ~ a[,c("age")], main=grade.names[3], type="n", lty=1, ylim=ylim, xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
## c.prime is wide, gray, solid
plot(c.prime[,c(log.grades[3])] ~ c.prime[,c("age")], main=NA, type="l",lwd=5, col="gray",  lty=1, ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
par(new=TRUE)
## a is the thin, black, solid
plot(a[,c(log.grades[3])] ~ a[,c("age")], main=NA, type="l", lwd=2, col="black", lty=1, ylim=ylim, xlim=c(0,100), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is thin, gray, solid
plot(b[,c(log.grades[3])] ~ b[,c("age")], main=NA, type="l", lwd=2, col="gray",lty=1,  ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
par(new=TRUE)
## c is thin, gray, dashed
#plot(c[,c(log.grades[3])] ~ c[,c("age")], main=NA, type="l", lwd=1, col="gray", lty=2,  ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)
#par(new=TRUE)
## c.prime thin, black, dashed
plot(c.prime[,c(log.grades[3])] ~ c.prime[,c("age")], main=NA, type="l",  lwd=2, col="black", lty=2, ylim=ylim, xlim=c(0,100), xlab=NA, ylab=NA)


## a = thin solid black
## b = thick dashed gray
## c = thin dashed gray
## c.prime = thin dashed black
  
################################################################################
## plot the #4 log grade results
grd <- 4
ylim <- c(0,15000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

grd <- 5
ylim <- c(0,5000)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)


################################################################################
## plot the #6 log grade results
grd <- 6
ylim <- c(0,500)
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=grade.names[grd], type="n", lty=1, ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
grid(lty=1)
par(new=TRUE)
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=5, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
par(new=TRUE)
## a is a thin, solid, black line
plot(a[,c(log.grades[grd])] ~ a[,c("age")], main=NA, lwd=2, lty=1, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab="Age, in Years", ylab=expression(feet^3))
par(new=TRUE)
## b is a thin, solid, gray line
plot(b[,c(log.grades[grd])] ~ b[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
##par(new=TRUE)
par(new=TRUE)
## c' is a thin dashed black line, over the thick solid gray line
plot(c.prime[,c(log.grades[grd])] ~ c.prime[,c("age")], main=NA, lwd=2, lty=2, col="black", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)

## c the a thin, solid, gray line
##plot(c[,c(log.grades[grd])] ~ c[,c("age")], main=NA, lwd=2, lty=1, col="gray", type="l", ylim=ylim, xlim=c(0,150), xlab=NA, ylab=NA)
##par(new=TRUE)

par(opar)




###################################################
### code chunk number 46: simulations.rnw:2275-2289
###################################################

cap <- paste("Volume metrics for simulations at maximum mean annual increment.") 
volumes <- c("col", "age", log.grades, "sm.vol")
mm3 <- max.mais[,volumes]

grade.names[2:5] <- paste("\\", grade.names[2:5], sep="")

names(mm3) <-  c("Model","Age",grade.names,"Total")

mm3$MAI <- mm3$Total / mm3$Age

latex(mm3, rowlabel=NULL, rowname=NULL, file="",caption=cap, caption.loc="top",
     label="tab:max-mais-volumes", digits=1, where='!h')



###################################################
### code chunk number 47: simulations.rnw:2434-2453
###################################################

hres <- res$rconifers
names(hres)[3] <- "rx" 

sql.file <- file("smc-yields.sql", "w")  
sql.columns <- names(hres)

for(i in 1:nrow(hres)) {
  sql.command <- 
      sprintf("insert into results (%s) values (%s);", 
              paste(as.vector(sql.columns), collapse=","),
              paste(hres[i,], collapse=",")) 
  cat(sql.command, file = sql.file, sep = "\n")
  
}

close(sql.file)




###################################################
### code chunk number 48: simulations.rnw:2678-2680
###################################################
system("rm -fr package-Ch7")
package.skeleton(name = "package-Ch7")

## this file contains the function originally from chapter 7. simulations.
## the function generate.sims( 1, blah, blah, blah ) returns a data.frame
## that contains various metrics associated with forest projections.
## volumes are computed using the method described simulations.rnw (section

## this file contains functions for
## the following functions are documented 
## generate.sims.Rd


## this function returns a data.frame object that represents
## the results of combined simulations.


generate.sims <- function( s,
                          ch.site=125.0, ch.pnba=1.0,
                          ##t.treats=c(200),
                          s.treats=c(0,0.25,0.50,0.75,1.0),
                          ##n.rep=1,
                          age.start=10, age.end=50)
{
  
  res <- NULL
  rcon.res <- NULL
  chambers.res <- NULL
  rcch.res <- NULL ## combined rconifers->25->chambers
  
  results <- data.frame()
  
    ## for each level of shrub removal
    for( j in 1:(length(s.treats)) ) {
      
      shrub.rem <- s.treats[j]
      
      ##        msg <- sprintf( "running t.treats[%d]=%f, s.treats[%d]=%f rep %d", i, t.treats[i], j, s.treats[j], k )
      msg <- sprintf( "running s.treats[%d]=%f", j, s.treats[j] )
      print( msg )
      
################################################################################
      ## 1) run the rconifers model first
      print( "running rconifers" )
      s3 <- s
      
      ## remove the competing vegetation
      s3$plants[s3$plants$sp.code == "CV",]$expf <-
        s3$plants[s3$plants$sp.code == "CV",]$expf * ( 1.0 - shrub.rem )        
      ## temporarily assign a sample.data object for annual simulation
      ## the stand is currently 3 years old
      s0 <- s3
      
      ## project the stand to age 153
      for( m in 1:200 ) {
        ## project sample.3 in one year intervals
        s1 <- project( s0, 1, control=list(rand.err=0,
                                rand.seed=0,
                                endemic.mort=1,
                                sdi.mort=1) )                  
        ## compute summaries for the projection and add it to the results data.frame
        ##rep.res <- data.frame( veg.ctrl=i, pct.crtl=j, rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="grey" )
        ##rep.res <- data.frame( veg.ctrl=i, pct.ctrl=j, rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers" )
        ##rep.res <- data.frame( veg.ctrl=s.treats[j], pct.ctrl=t.treats[i], rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers" )
        rep.res <- data.frame( veg.ctrl=s.treats[j], age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers" )
        print( rep.res )
        rcon.res <- rbind( rcon.res, rep.res )       
        s0 <- s1      
      }
      
      print( "done with the rconifers runs" )
        
################################################################################
      ## 2) run the chambers model second
      ## the projected stand is 30 years old and should have the nescessary 
      ## data to use to feed into the chambers-1980 model
      ##kings.site <- s1$site.index ## this needs to be converted to kings 1966 from smc @ 30        
      ## for each of the ages in the projection, you need to perform the following tasks:
      ## 1) construct a dummy sample.data object 
      ## 2) compute the volumes using sp.sums.2
      ## 3) append the resulting data.frame onto the results (res)
      print( "running chambers" )
      osr <- chambers.1980( ages=1:200, site=ch.site, pnba=ch.pnba )
      for( m in 10:200 ) {          
        s2.stem <- data.frame( plot=1,
                              sp.code="DF", 
                              d6=NA,
                              dbh=osr[m,]$qmd, 
                              tht=osr[m,]$tht,
                              cr=0.6,
                              n.stems=1,
                              expf=osr[m,]$expf,
                              crown.width=NA )
        s2.plot <- data.frame( plot=1,elevation=NA,slope=NA,
                              aspect=NA,whc=NA,map=NA,si30=ch.site )
        s2 <- list( plots=s2.plot, plants=s2.stem, age=m, x0=s$x0 )
        class( s2 ) <- "sample.data"          
        ## you need the volume here.
        ##rep.res <- data.frame( veg.ctrl=i, pct.crtl=j, rep=k, age=m,
        ##                      sp.sums.2( s2 )["DF",], col="black" )
        ##rep.res <- data.frame( veg.ctrl=s.treats[j], pct.ctrl=t.treats[i], rep=k, age=s2$age, sp.sums.2( s2 )["DF",], col="chambers" )
        ##rep.res <- data.frame( veg.ctrl=i, pct.ctrl=j, rep=k, age=s2$age, sp.sums.2( s2 )["DF",], col="chambers" )
        ## rbind the results to the larger results table.
        rep.res <- data.frame( veg.ctrl=s.treats[j], age=s2$age, sp.sums.2( s2 )["DF",], col="chambers" )
        print( rep.res )
        chambers.res <- rbind( chambers.res, rep.res )                 
      }
      print( "running chambers complete" )
      
      
      

################################################################################
        ## 3) run the rconifers model, and feed the results at age 25 into chambers
        ##s3 <- sample.3       
##       print( "running chambers from rconifers @ age 25" )
##         s3 <- s
##         ## remove the competing vegetation
##         s3$plants[s3$plants$sp.code == "CV",]$expf <-
##           s3$plants[s3$plants$sp.code == "CV",]$expf * ( 1.0 - shrub.rem )        
##         ## temporarily assign a sample.data object for annual simulation
##         ## the stand is currently 3 years old
##         s0 <- s3        
##         ## project the stand to age 25 (3+22)
##         for( m in 1:22 ) {          
##           ## project sample.3 in one year intervals
##           s1 <- project( s0, 1, control=list(rand.err=0,
##                                   rand.seed=0,
##                                   endemic.mort=1,
##                                   sdi.mort=1) )
##           ## compute summaries for the projection and add it to the results data.frame
##           ##rep.res <- data.frame( veg.ctrl=i, pct.crtl=j, rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="grey" )
##           ##rep.res <- data.frame( veg.ctrl=s.treats[j], pct.ctrl=t.treats[i], rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers->chambers" )
##           ##rep.res <- data.frame( veg.ctrl=s.treats[j], pct.ctrl=t.treats[i], rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers->chambers" )
##           ##rep.res <- data.frame( veg.ctrl=i, pct.ctrl=j, rep=k, age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers->chambers" )
##           rep.res <- data.frame( veg.ctrl=s.treats[j], age=s1$age, sp.sums.2( s1 )["DF",], col="rconifers->chambers" )
##           print( rep.res )
##           rcch.res <- rbind( rcch.res, rep.res )       
##           s0 <- s1      
##         }
##         ## compute the percent normal basal area to hand-off to the chambers model
##         rcch.pnba <- sp.sums.2( s1 )["DF",]$ba / chambers.1980.nba( ch.site, s1$age )
##         osr <- chambers.1980( ages=1:200, site=ch.site, pnba=rcch.pnba )
##         for( m in 26:200 ) {
          
##           s2.stem <- data.frame( plot=1,
##                                 sp.code="DF", 
##                                 d6=NA,
##                                 dbh=osr[m,]$qmd, 
##                                 tht=osr[m,]$tht,
##                                 cr=0.6,
##                                 n.stems=1,
##                                 expf=osr[m,]$expf,
##                                 crown.width=NA )
##           s2.plot <- data.frame( plot=1,elevation=NA,slope=NA,
##                                 aspect=NA,whc=NA,map=NA,si30=site )
##           s2 <- list( plots=s2.plot, plants=s2.stem, age=m, x0=s1$x0 )
##           class( s2 ) <- "sample.data"
          
##           ## compute the summaries for the sample.data object.
##           ##rep.res <- data.frame( veg.ctrl=i, pct.crtl=j, rep=k, age=m,
##           ##                      sp.sums.2( s2 )["DF",], col="black" )
## ##           rep.res <- data.frame( veg.ctrl=s.treats[j], pct.ctrl=t.treats[i], rep=k, age=s2$age,
## ##                                 sp.sums.2( s2 )["DF",], col="rconifers->chambers" )
##           ##rep.res <- data.frame( veg.ctrl=i, pct.ctrl=j, rep=k, age=s2$age, sp.sums.2( s2 )["DF",], col="rconifers->chambers" )
##           rep.res <- data.frame( veg.ctrl=s.treats[j], age=s2$age,
##                                 sp.sums.2( s2 )["DF",], col="rconifers->chambers" )
##           print( rep.res )

##           ## rbind the results to the larger results table.
##           rcch.res <- rbind( rcch.res, rep.res )                 
##         }
        
##      } ## done with the replications        
      ## for the current level of shrub removal    
    } ## done with the different shrub removal levels
##  } ## done with the different pct thinning levels
  
  ## you now have chambers results

  rcon.res$mai <- rcon.res$sm.vol / rcon.res$age
  chambers.res$mai <- chambers.res$sm.vol / chambers.res$age
##  rcch.res$mai <- rcch.res$sm.vol / rcch.res$age
  
  ## only return the data.frames for which you request data
  res$rconifers <- rcon.res[rcon.res$age >= age.start & rcon.res$age <= age.end,]
  res$chambers <- chambers.res[chambers.res$age >= age.start & chambers.res$age <= age.end,]
  ##res$rcch <- rcch.res[rcch.res$age >= age.start & rcch.res$age <= age.end,]

  ## only return the results where the two are the same length.
  ## for each of the runs, go back and 
  
  res
}

# for comparison to excel version

comment <- "This run is for comparison with the Excel version"


# inputfile
# just like in R, all comments start with #.
# order of parameters does not matter
# expressions allowed

# Comments must be Latex-friendly; ie
#       encapsulate math statements with $$.
#       don't use &

# labels
startage <- 1             ; c.startage <-     ""  
endage <- 60              ; c.endage <-       ""     
initialyear <- 1989       ; c.initialyear <-  "start in year"        
initialmonth <- 1         ; c.initialmonth <- "and january"           
yearplanted <- 1988       ; c.yearplanted <-  "for calculation of stand age"
monthplanted <- 1         ; c.monthplanted <- " ... "          
sitename <- " "           ; c.sitename <-     "a label for the site"       
speciesname  <- " "       ; c.speciesname  <- "and for the species"


# initial values:
aswi <- 999            ;  c.aswi <-    "initial available soil water (mm?)" 
wfi <- 7               ;  c.wfi <-     "initial foliage mass"
wri <- 2               ;  c.wri <-     "initial root mass"
wsi <- 5               ;  c.wsi <-     "initial stem mass"
stemnoi <- 1111        ;  c.stemnoi <- "initial stem number"


# site parameters
lat <- -26             ;  c.lat <-       "latitude (NH positive)"
maxasw <- 200          ;  c.maxasw <-    "maximum available soil water (SWHC)"
minasw <- 0            ;  c.minasw <-    "minimum available soil water"   
fr <- 1                ;  c.fr <-        "fertility rating"
soilclass <- "CL"      ;  c.soilclass <- "soil type"
swconst <- NA          ;  c.swconst <-   "soil water release parameters: leave NA when soilclass given."
swpower <- NA          ;  c.swpower <-   ""        



# Parameters
maxage <- 50 ;   c.maxage <- "Determines rate of physiological decline of forest"   
sla0 <- 4    ;   c.sla0 <- "Specific leaf area at age 0 ($m^2$/kg)"
sla1 <- 4    ;   c.sla1 <- "Specific leaf area for mature trees ($m^2$/kg)" 
tsla <- 2.5  ;   c.tsla <- "stand age (years) for SLA = (SLA0+SLA1)/2"


# fullcanage deleted: was not used in 3PG latest version


k <- 0.5          ; c.k <-    "Radiation extinction coefficient"
gammafx <- 0.03   ; c.gammafx <- "Coefficients in monthly litterfall rate"
gammaf0 <- 0.03   ; c.gammaf0 <- ""                       
tgammaf <- 24     ; c.tgammaf <- ""                        
rttover <- 0.015  ; c.rttover <- "Root turnover rate per month"
swconst0 <- 0.7   ; c.swconst0 <- "SW constants"
# are 0.7 for sand, 0.6 for sandy-loam, 0.5 for clay-loam, 0.4 for clay
swpower0 <- 9     ; c.swpower0 <- "Powers in the eqn for SW modifiers"
# are 9 for sand,  7 for sandy-loam, 5 for clay-loam and 3 for clay
maxintcptn <- 0.15; c.maxintcptn <- "Max proportion of rainfall intercepted by canopy"
laimaxintcptn <- 0; c.laimaxintcptn <- "LAI required for maximum rainfall interception"
maxcond <- 0.02   ; c.maxcond <- "Maximum canopy conductance (gc, m/s)"  
laigcx <- 3.33    ; c.laigcx <- "LAI required for maximum canopy conductance"


blcond <- 0.2     ; c.blcond <- "Canopy boundary layer conductance, assumed constant"
coeffcond <- 0.05 ; c.coeffcond <- "Determines response of canopy conductance to VPD"
y <- 0.47         ; c.y <- "Assimilate use efficiency"
tmax <- 32        ; c.tmax <- "Critical biological temperatures: max, min"
tmin  <- 2        ; c.tmin  <- ""                     
topt <- 20        ; c.topt <-  "and optimum. Reset if necessary/appropriate"
kf <- 1           ; c.kf <- "Number of days production lost per frost days"
pfs2 <- 1         ; c.pfs2 <- "Foliage:stem partitioning ratios for D = 2cm"
pfs20 <- 0.15     ; c.pfs20 <- "and D = 20cm"
stemconst <- 0.095; c.stemconst <- "Stem allometric parameters"
stempower <- 2.4  ; c.stempower <-  ""                         
prx <- 0.8        ; c.prx <-   "Maximum fraction of NPP to roots" 
prn <- 0.25       ; c.prn <-   "Minimum fraction of NPP to roots"
m0 <- 0           ; c.m0 <-    "Value of m when FR = 0"
fn0 <- 1          ; c.fn0 <-   "Value of fN when FR = 0"
alpha <- 0.055    ; c.alpha <- "Canopy quantum efficiency"
wsx1000 <- 100    ; c.wsx1000 <- "Max tree stem mass (kg) likely in mature stands  of 1000 trees/ha" 
thinpower <- 3/2  ; c.thinpower <- "exponent in the self-thinning routine."
fracbb0 <- 0.15   ; c.fracbb0 <-  "branch and bark fraction at age 0 ($m^2$/kg)"
fracbb1 <- 0.15   ; c.fracbb1 <- "branch and bark fraction for mature trees ($m^2$/kg)"
tbb <- 1.5        ; c.tbb <- "stand age (years) for fracBB = (fracBB0+fracBB1)/2"
density <- 0.5    ; c.density <- "basic density (t/m3)"


mf <- 0           ; c.mf <- "Leaf mortality fraction"
mr <- 11 / 54     ; c.mr <- "Root mortality fraction"
ms <- 11 / 54     ; c.ms <- "Stem mortality fraction"


rage <- 0.95      ; c.rage <- "parameters in the age modifier."
nage <- 4         ; c.nage <- ""

# $Id: interval_summary.R 169 2006-01-05 05:02:44Z andrewr $

rm(list=ls())

load("../scripts/booted_srs.RData")
require(lattice)

opar <- par(mfrow=c(3,3), mar=c(5,4,0,0))
for (i in 1:length(levels(plots$forest))) {
  plot(density(plots$ba[plots$forest==unique(plots$forest)[i]]),
       main="", ylab="",
       xlab=paste("National forest ID:", unique(plots$forest)[i]))
}
par(opar)

opar <- par(mfrow=c(3,3), mar=c(5,4,0,0))
for (i in 1:length(levels(plots$forest))) {
  qqnorm(plots$ba[plots$forest==unique(plots$forest)[i]],
         main="", ylab="",
         xlab=paste("National forest ID:", unique(plots$forest)[i]))
  qqline(plots$ba[plots$forest==unique(plots$forest)[i]])
}
par(opar)

xyplot(contained ~ n | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=overall.results)

xyplot(length ~ n | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=overall.results)

xyplot(contained ~ length | interval,
       panel = function(x, y) {
         panel.xyplot(x, y, type="l")
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=overall.results)

xyplot(length ~ n | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=5,
       title="Nearest National Forest", space="top"),
       xlab="Sample size, n",
       ylab="Mean interval length ()",
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=results)

xyplot(contained ~ n | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=5,
       title="Nearest National Forest", space="top"),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=results)

xyplot(contained ~ length | interval,
       groups=forest,
       auto.key = list(points = TRUE, columns=5,
       title="Nearest National Forest", space="top"),
       panel = function(x, y, subscripts, groups) {
         for (i in 1:length(levels(results$forest))) {
           panel.xyplot(x[groups[subscripts]==levels(results$forest)[i]],
                        y[groups[subscripts]==levels(results$forest)[i]],
                        col=trellis.par.get()$superpose.line$col[i],
                        type="l")
         }
         panel.abline(h=0.95, col="darkgrey")
       },
       index.cond=list(c(1:5,7:9,6,10:12)),
       data=results)

#	$Id: schedpack.r 4519 2010-12-19 23:32:13Z hamannj $	

## this is a set of functions to help with the R book
## for harvest scheduling.

##    No.   Row name   St   Activity     Lower bound   Upper bound    Marginal
## ------ ------------ -- ------------- ------------- ------------- -------------
##      1 hvol1        NS             0             0             =       78.3347
##      2 hvol2        NS             0             0             =      -14.3923
##      3 hvol3        NS             0             0             =      -104.408

get.row.report <- function( lp ) {
  
  ## get the number of columns
#  nrows <- Get_num_rows( lp )
  nrows <-  getNumRowsGLPK(lp)  
  ## build the table for the rows
  r <- NULL
  i <- NULL
  for( i in 1:nrows ) {
    r <- rbind( r,
               as.data.frame( cbind( getRowNameGLPK(lp, i),
                                     getRowStatGLPK( lp, i ),
                                     getRowPrimGLPK( lp, i ),
                                     getRowLowBndGLPK( lp, i ),
                                     getRowUppBndGLPK( lp, i ),
                                     getRowDualGLPK( lp, i )
                                    )
                             )
               )


  }
  
  
#  names(r) <- c("name","status","prim","lb","ub","dual","strerr" )
  names(r) <- c("name","status","prim","lb","ub","dual")  
  rownames( r ) <- 1:nrows

  r
}

  

#    No. Column name  St   Activity     Lower bound   Upper bound    Marginal
# ------ ------------ -- ------------- ------------- ------------- -------------
#      1 x(1)         B        109.153             0               
#      2 x(2)         NL             0             0                    -76.3559
#      3 x(3)         B            200             0               


get.col.report <- function( lp ) {

  ncols <- getNumColsGLPK( lp )

  ## build the table for the columns
  c <- NULL
  i <- NULL
  for( i in 1:ncols ) {
    c <- rbind( c,
               as.data.frame( cbind( I(getColNameGLPK( lp, i ) ),
                                     I(getColStatGLPK( lp, i ) ),
                                     I(getColPrimGLPK( lp, i ) ),
                                     I(getObjCoefGLPK( lp, i ) ),
                                     I(getColLowBndGLPK( lp, i ) ),
                                     I(getColUppBndGLPK( lp, i ) ),
                                     I(getColDualGLPK( lp, i ) ),
#                                     I(getCbindGLPK( lp, i ) ),
# Error in routine getCbindGLPK reported to Gabriel Gelius-Dietrich <geliudie@uni-duesseldorf.de> March 24th                                      
                                     I(getColTypeGLPK( lp, i ) )
                                    )
                             )
               )
print(i)
print(c)
    }
  
  
#  names(c) <- c("name","status","activity","coef", "lb","ub","dual","b_ind","type" )
  names(c) <- c("name","status","activity","coef","lb","ub","dual","type" )
  
#  colnames( c ) <- 1:ncols
#  Error in `colnames<-`(`*tmp*`, value = 1:48) : 
#    'names' attribute [48] must be the same length as the vector [7]

  c
}

# $Id: ratio_test.R 172 2006-01-10 05:16:07Z andrewr $

rm(list=ls())

plots <- read.csv("../data/fia_plots.csv")

plots$forest <- factor(plots$forest)

reps <- 2000
reps <- 2

n.s <- c(16, 32, 64, 96, 128)
n.s <- c(16, 32)

ls()

## The following code extracts the plot-level estimates from the FIA database.

#load("/home/andrewr/1.research/FVS\ Refit/fia.wo.VJAN09-06.RData")

#dim(fia.wo)

#names(fia.wo)

#fia.wo[1:10,]

#plot(fia.wo$DBH[1:100], fia.wo$BA[1:100])
#plot(fia.wo$DBH[1:100], fia.wo$BA[1:100]*fia.wo$TF[1:100])

plots <- aggregate(x=list(ba=fia.wo$SBA,
                    ht=fia.wo$HT),
                   by=list(forest=fia.wo$FORCODE,
                     plot=fia.wo$PLOT),
                   FUN=mean, na.rm=T)

#table(plots$forest)

#xyplot(ba ~ ht | forest, data=plots)

#write.csv(plots, file="../data/fia_plots.csv")

require(boot)
require(survey)

boot.ratio <- function(data, index) {
  y <- data$ba
  x <- data$ht
  R <- sum(y[index]) / sum(x[index])
  c(R,
    (var(y[index]) +
     R^2*var(x[index]) -
     2 * R * cov(y[index], x[index])) / length(x))
}

within <- function(conf, x)
  (x >= conf[length(conf)-1] & x < conf[length(conf)])
int.above <- function(conf, x)
  (x >= conf[length(conf)])
int.below <- function(conf, x)
  (x < conf[length(conf)-1])
int.length <- function(conf)
  conf[length(conf)] - conf[length(conf)-1]

## Sample with replacement 

store <- 0

ba.means <- tapply(plots$ba, plots$forest, mean)
ht.means <- tapply(plots$ht, plots$forest, mean)
  
total.reps <- reps * length(n.s) * length(levels(plots$forest))

interval.lengths <- list(rep(NA, total.reps))
interval.coverage <- list(rep(NA, total.reps))
interval.above <- list(rep(NA, total.reps))
interval.below <- list(rep(NA, total.reps))

for (j in 1:length(unique(n.s))) {            # j <- 1
  n <- n.s[j]
  for (i in 1:length(levels(plots$forest))) { # i <- 1
    for (b in 1:reps) {                       # b <- 1
      store <- store + 1
      indices <- sample(1:table(plots$forest)[i], size=n, replace=TRUE)
      the.sample <-
        plots[plots$forest==levels(plots$forest)[i],][indices,]
      boot.object <- boot(the.sample, boot.ratio, R=1999)
      ci <- boot.ci(boot.object)
      log.ci <- boot.ci(boot.object, type=c("norm","basic","stud"),
                        h=log, hdot=function(x) 1/x, hinv=exp)
      sqrt.ci <- boot.ci(boot.object, type=c("norm","basic","stud"),
                        h=sqrt, hdot=function(x) 2*x, hinv=function(x) x^2)

      R <- sum(the.sample$ba)/sum(the.sample$ht)
      classical <- R * ht.means[i] + c(-1,1) * qt(p=0.975, df=n-1) * 
                    sqrt((var(the.sample$ba) + R^2*var(the.sample$ht) -
                          2 * R * cov(the.sample$ba, the.sample$ht)) / n)
      
      ratio.lin <- svydesign(id=~1, data=the.sample, weights=1)
      ratio.lin.hat <- svyratio(~ba, ~ht, design=ratio.lin)
      lin.int <-
        matrix(unlist(predict(ratio.lin.hat, total=ht.means[i], se=TRUE)),
               nrow=1, byrow=TRUE) %*%
                 matrix(c(1,1,c(-1,1) * qt(p=0.975, df=n-1)),
                        nrow=2, byrow=TRUE)
          # Yes, I really thought that this would be compact!
      ratio.jack <- as.svrepdesign(ratio.lin)
      ratio.jack.hat <- svyratio(~ba, ~ht, design=ratio.jack)
      jack.int <-
        matrix(unlist(predict(ratio.jack.hat, total=ht.means[i], se=TRUE)),
               nrow=1, byrow=TRUE) %*%
                 matrix(c(1,1,c(-1,1) * qt(p=0.975, df=n-1)),
                        nrow=2, byrow=TRUE)
      
      interval.coverage[[store]] <-
        c(within(classical, ba.means[i]),
          within(lin.int, ba.means[i]),
          within(jack.int, ba.means[i]),
          as.numeric(lapply(ci[4:8], within, x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(log.ci[4:6], within, x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(sqrt.ci[4:6], within, x=ba.means[i]/ht.means[i])))
      interval.lengths[[store]] <- 
        c(int.length(classical),
          int.length(lin.int),
          int.length(jack.int),
          as.numeric(lapply(ci[4:8], int.length)) * ht.means[i],
          as.numeric(lapply(log.ci[4:6], int.length)) * ht.means[i],
          as.numeric(lapply(sqrt.ci[4:6], int.length)) * ht.means[i])
      interval.above[[store]] <- 
        c(int.above(classical, ba.means[i]),
          int.above(lin.int, ba.means[i]),
          int.above(jack.int, ba.means[i]),
          as.numeric(lapply(ci[4:8], int.above, x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(log.ci[4:6], int.above,
                            x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(sqrt.ci[4:6], int.above,
                            x=ba.means[i]/ht.means[i])))
      interval.below[[store]] <- 
        c(int.below(classical, ba.means[i]),
          int.below(lin.int, ba.means[i]),
          int.below(jack.int, ba.means[i]),
          as.numeric(lapply(ci[4:8], int.below, x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(log.ci[4:6], int.below,
                            x=ba.means[i]/ht.means[i])),
          as.numeric(lapply(sqrt.ci[4:6], int.below,
                            x=ba.means[i]/ht.means[i])))
    }
    cat("\nFinished simulation for Forest", levels(plots$forest)[i],
        "for n =", n, "(", reps, "reps )")
  }
  cat("\n")
}

output <- 
  expand.grid(interval=c("classical", "linearized","jackknife",
                "normal", "basic", "studentized", "percentile", "bca",
                "log.normal", "log.basic", "log.studentized",
                "sqrt.normal", "sqrt.basic", "sqrt.studentized"),
              replicate=1:reps,
              forest = levels(plots$forest),
              n = n.s)

output$length <- unlist(interval.lengths)
output$above <- unlist(interval.above) 
output$coverage <- unlist(interval.coverage) 
output$below <- unlist(interval.below) 

results <-
  aggregate(x=list(length=output$length,
              low=output$above,
              contained=output$coverage,
              high=output$below),
            by=list(interval=output$interval,
              forest=output$forest,
              n=output$n),
            FUN=mean)

results$scaled <- results$length /
  qt(results$contained, df=as.numeric(as.character(results$n))-1) *
  qt(0.95, df=as.numeric(as.character(results$n))-1) 

results$overCover <-
  (qt(0.95, df=as.numeric(as.character(results$n))-1) /
   qt(results$contained, df=as.numeric(as.character(results$n))-1))^2

results.tab <- results

results.tab$length <- format(results$length, dig=3)
results.tab$low <- format(results$low, dig=3)
results.tab$contained <- format(results$contained, dig=3)
results.tab$high <- format(results$high, dig=3)
results.tab$scaled <- format(results$scaled, dig=3)
results.tab$overCover <- format(results$overCover, dig=3)

results.tab[1:12,]

overall.results <-
  aggregate(x=list(length=results$length,
              low=results$low,
              contained=results$contained,
              high=results$high),
            by=list(interval=results$interval,
              n=results$n),
            FUN=mean)

overall.results$scaled <- overall.results$length /
  qt(overall.results$contained,
     df=as.numeric(as.character(overall.results$n))-1) *
  qt(0.95, df=as.numeric(as.character(overall.results$n))-1) 

overall.results$overCover <-
  (qt(0.95, df=as.numeric(as.character(overall.results$n))-1) /
   qt(overall.results$contained,
      df=as.numeric(as.character(overall.results$n))-1))^2

overall.results.tab <- overall.results

overall.results.tab$length <- format(overall.results$length, dig=3)
overall.results.tab$low <- format(overall.results$low, dig=3)
overall.results.tab$contained <- format(overall.results$contained, dig=3)
overall.results.tab$high <- format(overall.results$high, dig=3)
overall.results.tab$scaled <- format(overall.results$scaled, dig=3)
overall.results.tab$overCover <- format(overall.results$overCover, dig=3)

overall.results.tab

save(results, overall.results, plots, "booted_ratio.RData")

system("./closure")

## These functions are used in Chapter 3

ht.fvs.ni.ft <- function(spp, dbh.in) {
  # Parameters from Wykoff et al. (1982)
  ht.params <-
    data.frame(species = c("WP", "WL", "DF", "GF", "WH", "WC", "LP", "ES",
                 "SF", "PP", "MH"),
               c0 = c(5.19988, 4.97407, 4.81519, 5.00233, 4.97331, 4.89564,
                 4.62171, 4.92190, 4.76537, 4.92880, 4.77951),
               c1 = c(-9.26718, -6.78347, -7.29306, -8.19365, -8.19370,
                 -8.39057, -5.32481, -8.30289, -7.61062, -9.32795, -9.31743))
  dimensions <- as.data.frame(dbh.in)
  dimensions$this.order <- c(1:length(spp))
  dimensions$species <- spp
  dimensions <- merge(x=dimensions, y=ht.params, all.x=T, all.y=F)
  dimensions <- dimensions[order(dimensions$this.order, decreasing=F),]
  4.5 + exp(dimensions$c0 + dimensions$c1 / (dimensions$dbh.in + 1))
}


ht.fvs.ni.m <- function(spp, dbh.cm) 
  ht.fvs.ni.ft(spp, dbh.cm/2.54) * 0.3048

vol.fvs.ni.bdft <- function(spp, dbh.in, ht.ft) {
  # Parameters from Wykoff et al. (1982)
  bf.params <-
    data.frame(species = c("WP", "WL", "DF", "GF", "WH", "WC", "LP", "ES",
                 "SF", "PP", "MH"),
               b0.small = c(26.729, 29.790, 25.332, 34.127, 37.314, 10.472,
                 8.059, 11.851, 11.403, 50.340, 37.314),
               b1.small = c(0.01189, 0.00997, 0.01003, 0.01293, 0.01203,
                 0.00878, 0.01208, 0.01149, 0.01011, 0.01201, 0.01203),
               b0.large = c(32.516, 85.150, 9.522, 10.603, 50.680, 4.064,
                 14.111, 1.620, 124.425, 298.784, 50.680),
               b1.large = c(0.01181, 0.00841, 0.01011, 0.01218, 0.01306,
                 0.00799, 0.01103, 0.01158, 0.00694, 0.01595, 0.01306))
  b0 <- b1 <- rep(NA, length(spp))
  dimensions <- as.data.frame(cbind(dbh.in, ht.ft))
  dimensions$this.order <- c(1:length(spp))
  dimensions$species <- spp
  small <- !(dbh.in > 20.5)
  small[is.na(small)] <- TRUE # Arbitrary; dbh values are missing
  dimensions <- merge(x=dimensions, y=bf.params, all.x=T, all.y=F)
  dimensions <- dimensions[order(dimensions$this.order, decreasing=F),]
  b0[small] <- dimensions$b0.small[small]
  b1[small] <- dimensions$b1.small[small]
  b0[!small] <- dimensions$b0.large[!small]
  b1[!small] <- dimensions$b1.large[!small]
  b0 + b1 * dimensions$dbh.in^2 * dimensions$ht.ft
}

vol.fvs.ni.m3 <- function(spp, dbh.cm, ht.m) 
  vol.fvs.ni.bdft(spp, dbh.cm/2.54, ht.m/0.3048) *
  144 * # Board feet to cubic inches
  2.54^3 / # Cubic inches to cubic centimetres
  100^3  # Cubic centimetres to cubic metres

BAF.imp.2.met <- function(BAF.imp)
  BAF.imp *
  0.3048^2 * # Square feet to square metres
  2.47105381 # Acres to hectares
  
BAF.met.2.imp <- function(BAF.met)
  BAF.met /
  0.3048^2 / # Square metres to square feet
  2.47105381 # Hectares to acres

show.cols.with.na <- function(x) {
  if (class(x) != "data.frame") 
    stop("x must be a dataframe.\n")
  missing.by.column <- apply(is.na(x), 2, sum)
  if (sum(missing.by.column) == 0) {
    cat("No missing values.\n")
  } else {
    missing <- which(missing.by.column > 0)
    return(missing.by.column[missing])
  }
}

###################################################
### chunk number 1: Setup
###################################################
rm( list=ls() )

source( "../../scripts/schedpack.r" )

## this is the switch for the dan pickett problem, which can take considerable time
## this is the huge 'middle' matrix that contains the parameters for the 'model'
include.primary.dcm <- FALSE
include.primary.dcm <- TRUE

if(!require(xtable, quietly=TRUE)) install.packages("xtable")
if(!require(glpkAPI, quietly=TRUE)) 
  install.packages("glpkAPI", repos="http://cran.r-project.org")
if(!require(e1071, quietly=TRUE)) 
  install.packages("e1071", repos="http://cran.r-project.org", dependencies=TRUE)
if(!require(maptools, quietly=TRUE)) install.packages("maptools")
if(!require(lattice, quietly=TRUE)) install.packages("lattice")

options(width=65)


###################################################
### chunk number 2: 
###################################################

A <- 8     # A
T <- 6     # T
l <- 10    # \ell
P <- T * l # |T|



###################################################
### chunk number 3: 
###################################################

## \lambda(\mathcal{A}_{b})
stand.acres <- c(5000,5000,5000,5000,30000,20000,12000,2000)



###################################################
### chunk number 4: 
###################################################


leusch.ylds <- read.table("../../data/leuschner.txt", 
                          header = TRUE)



###################################################
### chunk number 5: 
###################################################

#library(glpk)
#leusch.lp <- lpx_create_prob()


#call glp_create_prob using glpkAPI initProbGLPK(ptrtype = "glpk_prob")

library(glpkAPI)
leusch.lp <- initProbGLPK()


###################################################
### chunk number 6: 
###################################################
#lpx_set_prob_name(leusch.lp, "leuschner")

#call glp_set_prob_name|assign (change) objective function name
#using GLPKAPI setProbNameGLPK(lp, pname = NULL)

setProbNameGLPK(leusch.lp, pname = "leushcner")


###################################################
### chunk number 7: 
###################################################

#lpx_set_obj_dir(leusch.lp, LPX_MAX)

#call glp_set_obj_dir
#Usage setObjDirGLPK(lp, lpdir)

setObjDirGLPK(leusch.lp, GLP_MAX)

###################################################
### chunk number 8: 
###################################################

#lpx_add_cols(leusch.lp, nrow( leusch.ylds ))

#Call GLPK function glp_add_cols. 
#Usage addColsGLPK(lp, ncols)

addColsGLPK(leusch.lp, nrow( leusch.ylds ))

###################################################
### chunk number 9: 
###################################################

leusch.ylds$dv <- paste("a", 
                        leusch.ylds$stand, 
                        leusch.ylds$period, 
                        sep="")
head(leusch.ylds)



###################################################
### chunk number 10: 
###################################################

for( t in 1:nrow(leusch.ylds)) {
  label <- leusch.ylds[t,]$dv
#  lpx_set_col_name(leusch.lp, t, label)
  setColNameGLPK(leusch.lp, t, label)
  }


#Call glp_set_col_name. 
#Usage setColNameGLPK(lp, j, cname = NULL)


###################################################
### chunk number 11: 
###################################################
for(t in 1:nrow(leusch.ylds)) {
#  lpx_set_col_bnds(leusch.lp, t, LPX_LO, 0.0, 0.0)
  setColBndGLPK(leusch.lp, t, GLP_LO, 0.0, 0.0) 
}

#Call GLPK function glp_set_col_bnds. 
#Usage setColBndGLPK(lp, j, type, lb, ub)


###################################################
### chunk number 12: 
###################################################

for(t in 1:nrow( leusch.ylds)) {
  ## set the objective coefficient in period t = 0.0
  #lpx_set_obj_coef(leusch.lp, t, leusch.ylds[t,]$vol)
  setObjCoefsGLPK(leusch.lp, t, leusch.ylds[t,]$vol)
} #$

#calls the GLPK function glp_set_obj_coef
#Usage setObjCoefsGLPK(lp, j, obj_coef)

###################################################
### chunk number 13: 
###################################################
#lpx_add_rows(leusch.lp, length(stand.acres))

addRowsGLPK(leusch.lp, length(stand.acres))

#Calls the GLPK function glp_add_rows. 
#Usage addRowsGLPK(lp, nrows)

###################################################
### chunk number 14: 
###################################################
acre.consts <- t( kronecker(diag(length(stand.acres)), 
                            as.matrix( rep(1,6))))



###################################################
### chunk number 15: 
###################################################

val <- rep(1, 6)  # this is the value of the constraint



###################################################
### chunk number 16: 
###################################################

## loop over the stands to generate 
## the idx is the index for the set of rows
## we are trying to set the coefficients for
for(i in 1:length(stand.acres)) {

  idx <- rep(0, 6)  # this is the index of the col coeffs

  ## manually create the index using id
  ## and assign row i, column j stored as idx[id]
  id <- 1
  for(j in 1:ncol(acre.consts)) {
    if(acre.consts[i,j] != 0.0) {
      idx[id] <- j
      id <- id + 1
    }
  }  
 
  ## now, perform all three tasks within the same loop
  ## set the constraint row name
#  lpx_set_row_name(leusch.lp, i, paste("s", i, sep=""))
  
# Call GLPK function glp_set_row_name. 
#Usage setRowNameGLPK(lp, i, rname = NULL)
  setRowNameGLPK(leusch.lp, i, paste("s", i, sep=""))
  
  ## set the upper bound on the acres for this stand
  ## to make sure no more than stand.acres[i] are cut
  
#  lpx_set_row_bnds(leusch.lp, i, LPX_UP, 0.0, 
#                   stand.acres[i])
  # Call the GLPK function glp_set_row_bnds. 
  # Usage setRowBndGLPK(lp, i, type, lb, ub)
  
  setRowBndGLPK(leusch.lp, i, GLP_UP, 0.0, 
                     stand.acres[i])
  
  ## set matrix row coefficients (?glp_set_mat_row)
  ## idx = ???, row index = i, val = vector of 1's
#  lpx_set_mat_row(leusch.lp, i, length(val), idx, val) 
  
# Call GLPK function glp_set_mat_row. 
# Usage setMatRowGLPK(lp, i, len, ind, val)

  setMatRowGLPK(leusch.lp, i, length(val), idx, val)
}



###################################################
### chunk number 17: 
###################################################

# Call the GLPK function glp_get_num_rows. 
# Usage getNumRowsGLPK(lp)

#lpx_get_num_rows(leusch.lp)  
getNumRowsGLPK(leusch.lp)


# Call the GLPK function glp_get_num_rows. 
# Usage getNumColsGLPK(lp)

#lpx_get_num_cols(leusch.lp)
getNumColsGLPK(leusch.lp)


###################################################
### chunk number 18: 
###################################################

target.acres <- rep(14000, 6)



###################################################
### chunk number 19: 
###################################################


#Calls the GLPK function glp_add_rows. 
#Usage addRowsGLPK(lp, nrows)

#lpx_add_rows(leusch.lp, length(target.acres))
addRowsGLPK(leusch.lp, length(target.acres))


###################################################
### chunk number 20: 
###################################################

# Call GLPK function glp_set_row_name. 
#Usage setRowNameGLPK(lp, i, rname = NULL)

# Call the GLPK function glp_set_row_bnds. 
# Usage setRowBndGLPK(lp, i, type, lb, ub)

# Call GLPK function glp_set_mat_row. 
# Usage setMatRowGLPK(lp, i, len, ind, val)


val <- rep(1,8) ## a vector of 8 ones.

for(i in 1:length(target.acres)) {
  idx <- 
     as.numeric(rownames(subset(leusch.ylds, period == i)))
#  lpx_set_row_name(leusch.lp, 
  setRowNameGLPK(leusch.lp,
                   i + length(stand.acres), 
                   paste("tac", i, sep="" ))
#  lpx_set_row_bnds(leusch.lp, 
  setRowBndGLPK(leusch.lp,
                   i + length(stand.acres), 
#                   LPX_FX, 
                   GLP_FX,
                   target.acres[i], 
                   target.acres[i])
#  lpx_set_mat_row(leusch.lp, 
  setMatRowGLPK(leusch.lp,
                  i + length(stand.acres), 
                  length(val), 
                  idx, 
                  val)
}



###################################################
### chunk number 21: 
###################################################

# Call the GLPK function glp_get_num_rows. 
# Usage getNumRowsGLPK(lp)

#lpx_get_num_rows(leusch.lp)  
getNumRowsGLPK(leusch.lp)


# Call the GLPK function glp_get_num_rows. 
# Usage getNumColsGLPK(lp)

#lpx_get_num_cols(leusch.lp)
getNumColsGLPK(leusch.lp)




###################################################
### chunk number 22: 
###################################################

# Call the  GLPK function glp_simplex. 
# Usage solveSimplexGLPK(lp)

#lpx_simplex(leusch.lp)

solveSimplexGLPK(leusch.lp)


###################################################
### chunk number 23: 
###################################################

# Call the GLPK function glp_get_obj_val. 
# Usage getObjValGLPK(lp)

#leusch.obj <- lpx_get_obj_val(leusch.lp)
leusch.obj <- getObjValGLPK(leusch.lp)
leusch.obj 



###################################################
### chunk number 24: 
###################################################
leusch.col.rpt <- get.col.report(leusch.lp)


###################################################
### chunk number 25: 
###################################################
ac.per <- 
   matrix(as.numeric(as.matrix(I(leusch.col.rpt$activity))), 
          6, 8)
ac.per


###################################################
### chunk number 26: 
###################################################
rowSums(ac.per)


###################################################
### chunk number 27: 
###################################################
vol.cut.a <- c(199.4,197.4,203.4,211.4,217.6,195.4)



###################################################
### chunk number 28: leuschner-area-control
###################################################

##vol.cut.a <- c(199.400,197.400,203.400,211.400,217.600,195.400)
vol.cut.b <- c(203.79, 203.79, 203.79, 203.79, 203.79, 203.79)

par(las = 1)
plot(vol.cut.a,  ylim=c(190,220), type="b", lwd=2, lty=1,
     main="Woodflow",
     xlab="Cutting period", ylab="Woodflow (cf x 10^3)")
par( new=TRUE )
plot( vol.cut.b, ylim=c(190,220), type="l", lwd=2, 
     lty=1, pch=3, main=NA, xlab=NA, ylab=NA )

##abline( h=mean( vol.cut.a ) - 0.05 * mean( vol.cut.a ), lty=2 )
##abline( h=mean( vol.cut.a ) + 0.05 * mean( vol.cut.a ), lty=2 )

leg <- cbind( problem=c("Strict Area Control","Strict Volume Control"), 
             color=c(1,1) )
leg.text <- leg[,1]
leg.col <- leg[,2]
legend(1, 220, 
       leg.text, 
       ##fill=leg.col,
       cex=1.0, 
       col=leg.col, 
       ##pt.bg=leg.col,
       lwd=c(2,2),
       lty=c(1,1),
       pch=c(1,NA),
       ncol=1,
       title="Plan Types" )



###################################################
### chunk number 29: fig-leuschner-area-control
###################################################
ac.per <- NULL
coef.per <- NULL

ac.per <- 
  matrix(as.numeric(as.matrix(I(leusch.col.rpt$activity))), 
         6, 8)
ac.per

coef.per <- matrix(as.numeric(as.matrix(I(leusch.col.rpt$coef))), 6, 8)
coef.per

vol.cut.a <- c(rowSums(ac.per *  coef.per) / 1000)

#<- c(199.400,197.400,203.400,211.400,217.600,195.400)
vol.cut.b <- c(203.79, 203.79, 203.79, 203.79, 203.79, 203.79)

par(las = 1)
plot(vol.cut.a,  ylim=c(190,220), type="b", lwd=2, lty=1,
     main="Woodflow",
     xlab="Cutting period", ylab="Woodflow (cf x 10^3)")
par( new=TRUE )
plot( vol.cut.b, ylim=c(190,220), type="l", lwd=2, 
     lty=1, pch=3, main=NA, xlab=NA, ylab=NA )

##abline( h=mean( vol.cut.a ) - 0.05 * mean( vol.cut.a ), lty=2 )
##abline( h=mean( vol.cut.a ) + 0.05 * mean( vol.cut.a ), lty=2 )

leg <- cbind( problem=c("Strict Area Control","Strict Volume Control"), 
             color=c(1,1) )
leg.text <- leg[,1]
leg.col <- leg[,2]
legend(1, 220, 
       leg.text, 
       ##fill=leg.col,
       cex=1.0, 
       col=leg.col, 
       ##pt.bg=leg.col,
       lwd=c(2,2),
       lty=c(1,1),
       pch=c(1,NA),
       ncol=1,
       title="Plan Types" )



###################################################
### chunk number 30: 
###################################################
leusch.col.rpt[1,] 


###################################################
### chunk number 31: 
###################################################
leusch.row.rpt <- get.row.report(leusch.lp)
leusch.row.rpt



###################################################
### chunk number 32: 
###################################################

leusch.row.rpt[1,]



###################################################
### chunk number 33: 
###################################################

# Call the GLPK function glp_write_mps. 
# Usage writeMPSGLPK(lp, fmt, fname)

#lpx_write_mps(leusch.lp, "leuschner.mps")

writeMPSGLPK(leusch.lp, GLP_MPS_FILE, "leuschner.mps")

###################################################
### chunk number 34: write leush
###################################################

# Call the GLPK function glp_write_lp. 
# Usage writeLPGLPK(lp, fname)

#lpx_write_cpxlp(leusch.lp, "leuschner.xlp")

writeLPGLPK(leusch.lp, "leuschner.xlp")

###################################################
### chunk number 35: 
###################################################

# Call the GLPK function glp_delete_prob. Consult the GLPK documentation
# Usage delProbGLPK(lp)

#lpx_delete_prob(leusch.lp)

delProbGLPK(leusch.lp)
leusch.lp


###################################################
### chunk number 36: 
###################################################
system("rm -fr package-Ch9")
package.skeleton(name = "package-Ch9")

#	$Id: chambers-1980.r 3684 2010-03-17 19:38:33Z hamannj $	

## this file contains R interface functions to the chambers-1980.so shared library
## the library is a single set of implimentation (*.c) and header (*.h) file
## and is created by issuing the following commands into the bash prompt:
## $ R CMD SHLIB chambers_1980.c

## /****************************************************************************/
## /* Charles J. Chambers 1980. Emperical growth and yield tables for the      */
## /* Douglas-fir zone.  DNR Report No. 41.  Washington Department of Natural  */
## /* Resources. Olympia WA. 98504                                             */
## /****************************************************************************/

## /*
##   Emperical yield functions for Douglas-fir from:

##          Min  Max     Mean      SD
##   Age     9    99    38.05   17.35
##   SI     38   165   107.39   21.42 (Kings 1966)
##   TPA               178.06  111.48
##   BA                135.93   78.37
##   CFV              4463.1  3353.87
##   QMD                12.35    3.36
##   normBA              0.82    0.38

## */

## chambers.1980.nba <- function(site.index,total.age)
## chambers.1980.ntpa.dnr <- function(site.index,total.age,pnba)
## chambers.1980.ntpa <- function(site.index,total.age,pnba)
## chambers.1980.adbh.dnr <- function(site.index,total.age,pnba)
## chambers.1980.adbh <- function(site.index,total.age,pnba)
## chambers.1980.atarif <- function(site.index,total.age,pnba)
## chambers.1980.cubic.foot.volume <- function(site.index,total.age,pnba)
## chambers.1980.scribner.16.volume <- function(site.index,total.age,pnba)
## chambers.1980.scribner.32.volume <- function(site.index,total.age,pnba)
## chambers.1980.atdba <- function(dbh,ba)
## chambers.1980.bfcfr <- function(dbh,tarif)
## chambers.1980.smh <- function(site.index,total.age)
## chambers.1980.nbag <- function(total.age,basal.area,site.index)


chambers.1980.nba <- function(site.index,total.age) {
  ret.val <- .C("chambers_1980_normal_basal_area",
                as.double(site.index),
                as.double(total.age),
                pred.nba = as.double(0) )$pred.nba
  ret.val
}


chambers.1980.ntpa.dnr <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_normal_tpa_dnr",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                ntpa.dnr = as.double(0) )$ntpa.dnr
  ret.val
}

## void chambers_1980_normal_tpa(
##     double   *site_index,
##     double   *total_age,
##     double   *pnba,
##     double   *ntpa );

chambers.1980.ntpa <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_normal_tpa",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                ntpa = as.double(0) )$ntpa
  ret.val
}

## void chambers_1980_avg_dbh_dnr(
##     double *site_index,
##     double *total_age,
##     double *pnba,
##     double *avg_dbh_dnr );

chambers.1980.adbh.dnr <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_avg_dbh_dnr",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                adbh.dnr = as.double(0) )$adbh.dnr
  ret.val
}


## /* double chambers_1980_avg_dbh( */
## /*     double *site_index, */
## /*     double *total_age, */
## /*     double *pnba                  ); */

## void chambers_1980_avg_dbh(
##     double *site_index,
##     double *total_age,
##     double *pnba,
##     double *avg_dbh );

chambers.1980.adbh <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_avg_dbh",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                adbh = as.double(0) )$adbh
  ret.val
}

## /* double chambers_1980_average_stand_tarif( */
## /*     double *site_index,  */
## /*     double *total_age,  */
## /*     double *pnba                  ); */

## void chambers_1980_average_stand_tarif(
##     double *site_index,
##     double *total_age,
##     double *pnba,
##     double *avg_stand_tarif );

chambers.1980.atarif <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_average_stand_tarif",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                atarif = as.double(0) )$atarif
  ret.val
}


## /* double chambers_1980_cubic_foot_volume( */
## /*     double *site_index,  */
## /*     double *total_age,  */
## /*     double *pnba                  ); */

## void chambers_1980_cubic_foot_volume(
##     double *site_index,
##     double *total_age,
##     double *pnba,
##     double *cubic_foot_volume );


chambers.1980.cubic.foot.volume <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_cubic_foot_volume",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                cfv = as.double(0) )$cfv
##  print( ret.val )
  ret.val
}

##chambers.1980.cubic.foot.volume( 120.0, 50.0, 1.0 )




## /* these next two functions still don't check out */
## /* double chambers_1980_scribner_16_volume( */
## /*     double *site_index,  */
## /*     double *total_age,  */
## /*     double *pnba                  );  */

## void chambers_1980_scribner_16_volume(
##     double *site_index,
##     double *total_age,
##     double *pnba,
##     double *scibner_16_volume );

chambers.1980.scribner.16.volume <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_scribner_16_volume",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                s16vol = as.double(0) )$s16vol
  ret.val
}


## /* double chambers_1980_scribner_32_volume( */
## /*     double *site_index,  */
## /*     double *total_age,  */
## /*     double *pnba                  ); */

## void chambers_1980_scribner_32_volume(
##    double *site_index,
##    double *total_age,
##    double *pnba,
##    double *scribner_32_volume );

chambers.1980.scribner.32.volume <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_scribner_32_volume",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                s32vol = as.double(0) )$s32vol
  ret.val
}

## /* double chambers_1980_cubic_volume_dbh_basal_area( */
## /*     double   *dbh, */
## /*     double   *basal_area          ); */

## void chambers_1980_cubic_volume_dbh_basal_area(
##     double   *dbh,
##     double   *basal_area,
##     double *cubic_volume_dbh_basal_area);

chambers.1980.scribner.32.volume <- function(site.index,total.age,pnba) {
  ret.val <- .C("chambers_1980_scribner_32_volume",
                as.double(site.index),
                as.double(total.age),
                as.double(pnba),
                s32vol = as.double(0) )$s32vol
  ret.val
}

    

## /* double chambers_1980_average_tarif_dbh_basal_area( */
## /*     double   *dbh, */
## /*     double   *basal_area          ); */

## void chambers_1980_average_tarif_dbh_basal_area(
##     double   *dbh,
##     double   *basal_area,
##     double *average_tarif_dbh_basal_area );

chambers.1980.atdba <- function(dbh,ba) {
  ret.val <- .C("chambers_1980_average_tarif_dbh_basal_area",
                as.double(dbh),
                as.double(ba),
                atdba = as.double(0) )$atdba
  ret.val
}





## /* double chambers_1980_board_foot_cubic_foot_ratio( */
## /*     double   *dbh, */
## /*     double   *tarif               ); */

## void chambers_1980_board_foot_cubic_foot_ratio(
##     double   *dbh,
##     double   *tarif,
##    double *board_foot_cubic_foot_ratio);

chambers.1980.bfcfr <- function(dbh,tarif) {
  ret.val <- .C("chambers_1980_board_foot_cubic_foot_ratio",
                as.double(dbh),
                as.double(tarif),
                bfcfr = as.double(0) )$bfcfr
  ret.val
}



## /* double chambers_1980_stand_mean_height( */
## /*     double   *site_index, */
## /*     double   *total_age          ); */

## void chambers_1980_stand_mean_height(
##    double   *site_index,
##    double   *total_age,
##    double *stand_mean_height );

chambers.1980.smh <- function(site.index,total.age) {
  ret.val <- .C("chambers_1980_stand_mean_height",
                as.double(site.index),
                as.double(total.age),
                smh = as.double(0) )$smh
  ret.val
}

## /* double chambers_1980_net_basal_area_growth( */
## /*    double   *total_age, */
## /*    double   *basal_area, */
## /*    double   *site_index ); */

## void chambers_1980_net_basal_area_growth(
##    double   *total_age,
##    double   *basal_area,
##    double   *site_index,
##    double *net_basal_area_growth );

chambers.1980.nbag <- function(total.age,basal.area,site.index) {
  ret.val <- .C("chambers_1980_stand_mean_height",
                as.double(total.age),
                as.double(basal.area),
                as.double(site.index),
                nbag = as.double(0) )$nbag
  ret.val
}


## this function generates a data.frame that contain a few metrics of interest.
chambers.1980 <- function( ages=1:100, site=125.0, pnba=1.0 ) {
  ret.val <- matrix( 0, length(ages), 5 )
  for( i in ages ) {    
    res <- c( i, 
             chambers.1980.adbh(site,i,pnba), ## table 5, page 6
             chambers.1980.smh( site,i ), ## table 16, page 10
             chambers.1980.nba(site,i ), ## table 1, page 4
             chambers.1980.ntpa( site,i,pnba ) ## table 3, page 5
             )
    ret.val[i,] <- res
  }
  ret.val <- as.data.frame( ret.val )
  names( ret.val ) <- c("age","qmd","tht","ba","expf" )
  ret.val
}
